import rel, optparse, socket

MODULES = "app_proxy, django_hello_world, echo_server, get_url, hello_client, http_client, http_client2, httpd_hello_world, http_proxy, new_conn, op_callback_server, op_callback_test, op_test, stomp_test, websocket_test, websocket_proxy_test, wsgi_test"

def parse_input():
    parser = optparse.OptionParser('dez_test [MODULE]\nmodules: %s'%(MODULES))
    parser.add_option("-p", "--port", dest="port", default="8888", help="run test on PORT. default: 8888")
    parser.add_option("-d", "--domain", dest="domain", default="localhost", help="run test on DOMAIN. default: localhost")
    parser.add_option("-f", "--function", dest="function", default="main", help="run this FUNCTION in test module. default: main")
    parser.add_option("-e", "--event", dest="event", default="pyevent", help="(event listener) notification method. default: pyevent. options: %s"%str(rel.supported_methods)[1:-1])
    parser.add_option("-r", "--report", action="store_true", dest="report", default=False, help="(event listener) status report every 5 seconds (non-pyevent only)")
    parser.add_option("-v", "--verbose", action="store_true", dest="verbose", default=False, help="(event listener) verbose output")
    parser.add_option("-s", "--strict", action="store_true", dest="strict", default=False, help="(event listener) _only_ try specified notification method")
    parser.add_option("-t", "--threaded", action="store_true", dest="threaded", default=False, help="(event listener) enable GIL hack to run threaded pyevent 0.3 application")
    return parser.parse_args()

def error(msg):
    print "error:",msg
    print 'type "dez_test -h" for help'

def main():
    options, arguments = parse_input()
    if not arguments:
        return error('cannot run dez_test without [MODULE] argument.\n\nie: "dez_test echo_server"\n\navailable modules: %s'%(MODULES))
    testfile = arguments[0]
    rel_options = []
    if options.report:
        rel_options.append('report')
    if options.verbose:
        rel_options.append('verbose')
    if options.strict:
        rel_options.append('strict')
    if options.threaded:
        rel_options.append('threaded')
    rel.initialize([options.event],rel_options)
    try:
        testport = int(options.port)
    except:
        return error('non-integer port specified')
    testfunc = options.function
    try:
        testmod = __import__("dez.samples.%s"%testfile,fromlist=["dez","samples"])
    except ImportError:
        return error("invalid module specified")
    if not hasattr(testmod, testfunc):
        return error("invalid function specified")
    testdomain = options.domain
    print 'running %s:%s on http://%s:%s'%(testfile,testfunc,testdomain,testport)
    try:
        getattr(testmod,testfunc)(domain=testdomain,port=testport)
    except socket.error:
        print '\nPermission denied to use port %s. Depending on how your system is set up, you may need root privileges to run this test.'%(testport)


from dez.buffer import Buffer

class HTTPClientWriter(object):
    def __init__(self, conn):
        self.conn = conn

    def dispatch(self, request, cb, args):
        request.headers['Host'] = self.conn.addr[0]
        self.conn.write(request.render(), self.__request_written_cb, [cb, args])

    def __request_written_cb(self, cb, args):
        return cb(*args)

class HTTPClientRequest(object):
    def __init__(self):
        self.protocol = "HTTP/1.1"
        self.method = "GET"
        self.path = "/"
        self.headers = {}
        self.body = Buffer()

    def write(self, data):
        self.body += data
        self.headers['Content-Length'] = str(len(self.body))

    def render(self):
        output = "%s %s %s\r\n" % (self.method, self.path, self.protocol)
        output += "\r\n".join( [": ".join((key, val)) for (key, val) in self.headers.items() ])
        output += "\r\n\r\n"
        output += self.body.data
        return output

class HTTPClientRawRequest(object):
    def __init__(self):
        self.protocol = "HTTP/1.1"
        self.method = "GET"
        self.path = "/"


from dez.http.application import HTTPApplication

def simple_app(environ, start_response):
    """Simplest possible application object"""
    status = '200 OK'
    response_headers = [('Content-type','text/plain')]
    start_response(status, response_headers)
    return ['Hello world!\n']
    
def main(**kwargs):
    httpd = HTTPApplication(kwargs['domain'], kwargs['port'])
    httpd.add_wsgi_rule("/", simple_app)
    httpd.start()



from dez.http.application import HTTPApplication

def simple_app(environ, start_response):
    print 'POST DATA:'
    print environ['wsgi.input'].read()
    return []
    
def main(**kwargs):
    httpd = HTTPApplication(kwargs['domain'], kwargs['port'])
    httpd.add_wsgi_rule("/", simple_app)
    httpd.start()



import event
import dez.io
from dez.buffer import Buffer, B64ReadBuffer, B64WriteBuffer
from dez.json import decode
from dez.xml_tools import extract_xml, XMLNode

RBUFF = {True:B64ReadBuffer, False:Buffer}
WBUFF = {True:B64WriteBuffer, False:Buffer}
SCINT = 0.1

class Connection(object):
    id = 0
    def __init__(self, addr, sock, pool=None, b64=False):
        Connection.id += 1
        self.id = Connection.id
        self.pool = pool
        self.addr = addr
        self.sock = sock
        self.mode = None
        self.__write_queue = []
        self.__write_chunk = None
        self.__write_buffer = WBUFF[b64]()
        self.__read_buffer = RBUFF[b64]()
        self.__looping = False
        self.__mode_changed = False
        self.__release_timer = None
        self.__close_cb = None
        self.revent = None
        self.wevent = None
        if not self.pool:
            self.__start()

    def connect(self, timeout=5):
        self.connect_timer = event.timeout(timeout, self.__connect_timeout_cb)
        self.connect_event = event.write(self.sock, self.__connected_cb)

    def set_close_cb(self, cb, args=[]):
        self.__close_cb = (cb, args)

    def __start(self):
        self.wevent = event.write(self.sock, self.__write_ready)
        self.revent = event.read(self.sock, self.__read_ready)
        self.wevent.delete()

    def __connected_cb(self):
        self.connect_event.delete()
        self.connect_event = None
        self.connect_timer.delete()
        self.connect_timer = None
        self.__start()
        self.__ready()

    def __connect_timeout_cb(self):
        self.connect_timer.delete()
        self.connect_timer = None
        self.close()

    def soft_close(self, reason=""):
        if self.__write_chunk or self.__write_queue:
            return event.timeout(SCINT, self.soft_close, reason)
        self.close(reason)

    def close(self, reason=""):
        if self.revent:
            self.revent.delete()
            self.revent = None
        if self.wevent:
            self.wevent.delete()
            self.wevent = None
        self.sock.close()
        self.__clear_writes(reason)
        if self.pool:
            self.pool.connection_closed(self)
            self.pool = None
        if self.mode:
            self.mode.close(self.__read_buffer)
            self.mode = None
        if self.__close_cb:
            cb, args = self.__close_cb
            self.__close_cb = None
            if cb:
                cb(*args)

    def __clear_writes(self, reason=""):
        if self.__write_chunk:
            self.__write_chunk.error(reason)
            self.__write_chunk = None
        while self.__write_queue:
            self.__write_queue.pop(0).error(reason)

    def release(self, timeout=0):
        self.halt_read()
        self.__read_buffer.reset()
        self.__clear_writes("Connection freed")
        if not self.__release_timer:
            self.__ready()
        else:
            print 'pool', self.pool
            print 'release_timer', self.__release_timer
            raise "What?"

    def __release_timer_cb(self):
        self.__release_timer.delete()
        self.__release_timer = None
        self.pool.connection_available(self)

    def __ready(self):
        self.halt_read()
        self.__read_buffer.reset()
        self.__clear_writes("Connection freed")
        if self.pool:
            self.pool.connection_available(self)

    def write(self, data, cb=None, cbargs=[], eb=None, ebargs=[]):
        if data.__class__ == XMLNode:
            data = str(data)
        self.__write_queue.append(WriteChunk(data, cb, cbargs, eb, ebargs))
        if not self.wevent:
            raise ConnectionClosedException, "Connection %s is closed and can no longer write. Call Connection.set_close_cb for close notification."%self.id
        if not self.wevent.pending():
            self.wevent.add()

    def set_rmode_size(self, size, cb, args=[]):
        self.mode = SizeReadMode(size, cb, args)
        self.__mode_changed = True
        self.__start_read()

    def set_rmode_size_chunked(self, size, cb, args=[]):
        self.mode = SizeChunkedReadMode(size, cb, args)
        self.__mode_changed = True
        self.__start_read()

    def set_rmode_close(self, cb, args=[]):
        self.mode = CloseReadMode(cb, args)
        self.__mode_changed = True
        self.__start_read()

    def set_rmode_close_chunked(self, cb, args=[]):
        self.mode = CloseChunkedReadMode(cb, args)
        self.__mode_changed = True
        self.__start_read()

    def set_rmode_delimiter(self, delimiter, cb, args=[]):
        self.mode = DelimeterReadMode(delimiter, cb, args)
        self.__mode_changed = True
        self.__start_read()

    def set_rmode_json(self, cb, args=[]):
        self.mode = JSONReadMode(cb, args)
        self.__mode_changed = True
        self.__start_read()

    def set_rmode_xml(self, cb, args=[], silent_readerror=True):
        """
        silent_readerror
            the default setting (True) allows stray characters
            (spaces, line delimiters, etc) between valid nodes
        """
        self.mode = XMLReadMode(cb, args, silent_readerror)
        self.__mode_changed = True
        self.__start_read()

    def halt_read(self):
        self.mode = None

    def __start_read(self):
        if self.mode == None:
            raise Exception("NoModeSet"), "First set a read mode"
        self.__read("")

    # Keep a queue of things to write and their related callbacks
    # if we just finished writing some segment, call its callback
    # if there are no more segments, then stop
    def __write_ready(self):
        if self.__write_buffer.empty():
            if self.__write_chunk:
                self.__write_chunk.completed()
                self.__write_chunk = None
            if not self.__write_queue:
                return None
            self.__write_chunk = self.__write_queue.pop(0)
            self.__write_buffer.reset(self.__write_chunk.data)
        try:
            bsent = len(self.__write_buffer)
            self.sock.send(self.__write_buffer.get_value())
            self.__write_buffer.move(bsent)
            return True
        except dez.io.socket.error, msg:
            self.close(reason=str(msg))
            return None

    # This function is far trickier than it originally seemed.
    # the __looping and __mode_changed variables are required to keep
    # from ignoring mode changes in the middle of the loop.
    def __read(self, data):
        self.__read_buffer += data
        if self.__looping:
            return
        self.__looping = True
        while self.mode:
            self.__mode_changed = False
            if not self.mode.ready(self.__read_buffer):
                break
            reschedule = self.mode.send_data(self.__read_buffer)
            if not reschedule and not self.__mode_changed:
                self.mode = None
        self.__looping = False

    def __read_ready(self):
        try:
            data = self.sock.recv(dez.io.BUFFER_SIZE)
        except dez.io.socket.error:
            self.close()
            return None
        if not data:
            self.close()
            return None
        self.__read(data)
        return True

class ConnectionClosedException(Exception):
    pass

class InvalidXMLException(Exception):
    pass

class WriteChunk(object):
    def __init__(self, data, cb=None, args=[], eb=None, ebargs=[]):
        self.data = data
        self.cb = cb
        self.args = args
        self.eb = eb
        self.ebargs = ebargs

    def completed(self):
        if self.cb:
            self.cb(*self.args)

    def error(self, reason=None):
        if self.eb:
            self.eb(*self.ebargs)

class SizeReadMode(object):
    def __init__(self, size, cb, args):
        self.args = args
        self.size = size
        self.completed = False
        self.cb = cb

    def ready(self, buffer):
        return self.size <= len(buffer)

    def send_data(self, buffer):
        data = buffer.part(0, self.size)
        buffer.move(self.size)
        self.cb(data, *self.args)
        return False

    def close(self, buffer):
        pass

class SizeChunkedReadMode(object):
    def __init__(self, size, cb, args):
        self.size = size
        self.completed = False
        self.cb = cb
        self.args = args

    def ready(self, buffer):
        return len(buffer) > 0

    def send_data(self, buffer):
        data = buffer.part(0, min(self.size, len(buffer)))
        buffer.move(len(data))
        self.size -= len(data)
        self.cb(data, *self.args)
        if self.size == 0:
            self.cb("")
            return False
        return True

    def close(self, buffer):
        pass

class CloseReadMode(object):
    def __init__(self,  cb, args):
        self.completed = False
        self.cb = cb
        self.args = args

    def ready(self, buffer):
        return False

    def send_data(self, buffer):
        raise Exception("InvalidCall"), "How did this get called?"

    def close(self, buffer):
        self.cb(buffer.get_value(), *self.args)
        buffer.reset()

class CloseChunkedReadMode(object):
    def __init__(self, cb, args):
        self.completed = False
        self.cb = cb
        self.args = args

    def ready(self, buffer):
        return len(buffer) > 0

    def send_data(self, buffer):
        data = buffer.get_value()
        buffer.reset()
        self.cb(data, *self.args)
        return True

    def close(self, buffer):
        if len(buffer) > 0:
            self.cb(buffer.get_value(), *self.args)
            buffer.reset()

class DelimeterReadMode(object):
    def __init__(self, delimiter, cb, args):
        self.completed = False
        self.cb = cb
        self.delimiter = delimiter
        self.args = args
        self.delim_index = -1

    def ready(self, buffer):
        self.delim_index = buffer.find(self.delimiter)
        return self.delim_index != -1

    def send_data(self, buffer):
        data = buffer.part(0, self.delim_index)
        buffer.move(self.delim_index + len(self.delimiter))
        self.delim_index = -1
        self.cb(data, *self.args)
        return True

    def close(self, buffer):
        pass

class JSONReadMode(object):
    def __init__(self, cb, args):
        self.completed = False
        self.cb = cb
        self.args = args
        self.checked_index = 0
        self.frame = None

    def ready(self, buffer):
        buff = buffer.get_value()
        if not buff:
            return False
        if buff[0] == "[":
            c = ']'
        elif buff[0] == "{":
            c = '}'
        else:
            self.checked_index = 0
            buffer.move(1)
            return self.ready(buffer)
        i = buffer.find(c, self.checked_index)
        while i != -1:
            try:
                self.frame = decode(buffer.part(0, i+1))
                buffer.move(i+1)
                self.checked_index = 0
                return True
            except:
                self.checked_index = i+1
                i = buffer.find(c, self.checked_index)
        return False

    def send_data(self, buffer):
        self.cb(self.frame, *self.args)
        self.frame = None
        return True

    def close(self, buffer):
        pass

class XMLReadMode(object):
    def __init__(self, cb, args, silent_readerror):
        self.completed = False
        self.cb = cb
        self.args = args
        self.checked_index = 0
        self.name = None
        self.name_count = 0
        self.frame = None
        self.silent = silent_readerror

    def ready(self, buffer):
        if not self.name:
            buff = buffer.get_value()
            if not buff:
                return False
            if buff[0] != "<":
                if not self.silent:
                    raise InvalidXMLException, 'Invalid first character in XML node: %s found instead of "<"'%buff[0]
                self.checked_index = 0
                buffer.move(1)
                return self.ready(buffer)
            if ">" not in buff:
                return False
            close_index = buff.index('>')
            if buff[close_index-1] == '/':
                self.frame = extract_xml(buff[:close_index+1])
                buffer.move(close_index+1)
                self.checked_index = 0
                return True
            self.name = buff[1:close_index].split(' ',1)[0]
            self.name_count = 1
            self.checked_index = close_index+1
        i = buffer.find(">", self.checked_index)
        while i != -1:
            if buffer.part(i-1-len(self.name),i+1) == "<"+self.name+">":
                self.name_count += 1
            elif buffer.part(i-2-len(self.name),i+1) == "</"+self.name+">":
                self.name_count -= 1
                if not self.name_count:
                    self.frame = extract_xml(buffer.part(0, i+1))
                    buffer.move(i+1)
                    self.checked_index = 0
                    self.name = None
                    return True
            self.checked_index = i+1
            i = buffer.find(">", self.checked_index)
        return False

    def send_data(self, buffer):
        self.cb(self.frame, *self.args)
        self.frame = None
        return True

    def close(self, buffer):
        pass



from base64 import b64encode, b64decode

class Buffer(object):
    ''' A Buffer object buffers text, and has two modes, 'index' and 'consume'

        In index mode, which is the default initial mode of a Buffer object,
        the buffer object keeps track of a position 'pos', which can be
        altered by calling b.move, or by b.exhaust, which moves 'pos' to the
        end of the file.

        In consume mode, a buffer is essentially just a proxy for its contained
        string b.data.

        If you experience memory leaks, make sure you don't have buffers sitting
        around in 'index' mode.
    '''
    def __init__(self, initial_data='', mode='consume'):
        self.mode = None
        self.pos = None
        self.data = initial_data
        self.set_mode(mode)

    def set_mode(self, mode):
        # No change
        if self.mode == mode:
            return
        # Switch from consume to index or initial mode is index
        elif mode == 'index':
            self.mode = 'index'
            self.pos = 0
        # Switch from index to consume
        elif mode == 'consume' and self.pos is not None:
            self.data = self.data[self.pos:]
            self.pos = None
            self.mode = 'consume'
        # initial mode is consume
        else:
            self.mode = 'consume'
            self.pos = None

    def find(self, marker, i=0):
        if self.mode == 'consume':
            return self.data.find(marker, i)
        elif self.mode == 'index':
            pos =  self.data.find(marker, i + self.pos)
            if pos == -1:
                return -1
            return pos - self.pos

    def __str__(self):
        return self.get_value()

    def get_value(self):
        ''' Return the data in consume mode, or the remainder of the data in
            index mode.
        '''
        if self.mode == 'consume':
            return self.data
        elif self.mode == 'index':
            return self.data[self.pos:]

    def get_full_value(self):
        ''' Returns the full value of the string, even in index mode. '''
        return self.data

    def exhaust(self):
        ''' In index mode, sets position to the end of the buffered data.  In
            consume mode, sets data to empty.
        '''
        if self.mode == 'consume':
            self.data = ''
        elif self.mode == 'index':
            self.pos = len(self.data)

    def reset_position(self):
        ''' Resets position to 0. '''
        if self.mode == 'index':
            self.pos = 0

    def reset(self, content = ''):
        ''' Resets data to empty, or an opional 'content' string, and position
            to 0.
        '''
        self.data = content
        if self.mode == 'index':
            self.pos = 0

    def empty(self):
        ''' Boolean; is true if the the buffer is empty '''
        if self.mode == 'consume':
            return len(self.data) == 0
        elif self.mode == 'index':
            return (len(self.data) - self.pos) == 0

    def move(self, i):
        if self.mode == 'consume':
            self.data = self.data[i:]
        elif self.mode == 'index':
            self.pos += i

    def part(self, start, end):
        if self.mode == 'consume':
            return self.data[start:end]
        elif self.mode == 'index':
            return self.data[self.pos + start: self.pos + end]

    def __contains__(self, data):
        if self.mode == 'consume':
            return data in self.data
        elif self.mode == 'index':
            return self.data.find(data, self.pos) != -1

    def __eq__(self, data):
        if self.mode == 'consume':
            return self.data == data
        elif self.mode == 'index':
            return self.data[self.pos:] == data

    def __len__(self):
        if self.mode == 'consume':
            return len(self.data)
        elif self.mode == 'index':
            return len(self.data) - self.pos

    def __get_slice__(self, start, end):
        return self.part(self, start, end)

    def __getitem__(self, key):
        return self.data[key]

    def __add__(self, add_data):
        ''' Add the passed-in string to the buffer '''
        self.data += add_data
        return self

class B64ReadBuffer(Buffer):
    ''' This works exactly like the Buffer class, except it
        reads base64-encoded strings separated by whitespace.
    '''
    def __init__(self, initial_data='', mode='consume'):
        self.raw_data = ''
        Buffer.__init__(self, initial_data, mode)

    def __add__(self, add_data):
        ''' Add the passed-in base64-encoded string to the pre-buffer '''
        s = add_data.find(' ')
        if s != -1:
            self.data += b64decode(self.raw_data+add_data[:s])
            add_data = add_data[s+1:]
            self.raw_data = ''
        self.raw_data += add_data
        return self

class B64WriteBuffer(Buffer):
    ''' This works exactly like the Buffer class, except
        get_value() returns a base64-encoded string
    '''
    def get_value(self):
        ''' Return the base64-encoded data in consume mode, or the base64-encoded remainder of the data in
            index mode.
        '''
        if self.mode == 'consume':
            return b64encode(self.data)+' '
        elif self.mode == 'index':
            return b64encode(self.data[self.pos:])+' '


from dez.op.server.server import OPServer
from orbited.op.message import OrbitMessage
import random,event

def deliver(d):
    event.timeout(random.random(),random.choice([d.success,d.failure]))

class TestApp(object):
    def __init__(self):
        self.connections = {}

    def connection_opened(self, conn):
        print 'connection_opened'
        conn.set_request_cb(self.request)
        self.connections[conn.id] = conn

    def connection_closed(self, conn):
        del self.connections[conn.id]

    def request(self, req):
        print 'request'
        if getattr(self,req.action.lower())(req):
            req.received()

    def welcome(self, req):
        return True

    def unwelcome(self, req):
        return True

    def callback(self, req):
        return True

    def connect(self, req):
        cid = req.headers["connection_id"]
        if cid in self.connections:
            print 'connection_id in use'
            req.error("'connection_id' %s in use"%cid)
            return False
        return True

    def send(self, req):
        msg = OrbitMessage(req.body, req.headers["recipients"], self.msg_outcome, [req])
        for r in msg.recipients:
            deliver(msg.single_recipient_message(r))
        return True

    def msg_outcome(self, message, frame):
        if message.success_recipients:
            frame.success(message.success_recipients)
        if message.failure_recipients:
            frame.failure(message.failure_recipients)

def main(**kwargs):
    app = TestApp()
    server = OPServer(kwargs['domain'], kwargs['port'])
    server.set_connect_cb(app.connection_opened)
    server.start()




from dez.http.client import HTTPClientRequest, HTTPClient
import event
import time

def main(**kwargs):
    event.initialize(["pyevent"])
    client = HTTPClient()
    for i in range(20):
#        client.get_url("http://www.google.com/", cb=response_cb)
#        client.get_url("http://www.google.com/", cb=response_cb)
#        client.get_url("http://www.google.com/", cb=response_cb)
#        client.get_url("http://www.google.com/", cb=response_cb)
        client.get_url("http://"+domain+":"+port+"/", cb=response_cb, cbargs=[i+1])
    event.signal(2, event.abort)
    event.dispatch()
    
def response_cb(response, i):
#    print 'woot
    print i, time.time(), response.status_line
#    print response.headers
#    print "#########"    
#    print "body len:", len(response.body)
#    event.abort()    



from request import HTTPClientRequest, HTTPClientWriter
from response import HTTPClientResponse, HTTPClientReader
from dez.network import SocketClient
import event

class HTTPClient(object):
    def __init__(self):
        self.client = SocketClient()
        self.id = 0
        self.requests = {}

    def get_url(self, url, method='GET', headers={}, cb=None, cbargs=(), eb=None, ebargs=(), body="", timeout=None):
        self.id += 1
        path, host, port = self.__parse_url(url)
        self.requests[self.id] = URLRequest(self.id, path, host, port, method, headers, cb, cbargs, eb, ebargs, body, timeout=timeout)
        self.client.get_connection(host, port, self.__conn_cb, [self.id], self.__conn_timeout_cb, [self.id])
    
    def __conn_timeout_cb(self, id):
        self.requests[id].timeout()
    
    def __conn_cb(self, conn, id):
        writer = HTTPClientWriter(conn)
        request = HTTPClientRequest()
        url_request = self.requests[id]
        request.headers.update(url_request.headers)
        request.method = url_request.method
        request.host = url_request.host
        request.path = url_request.path
        request.port = url_request.port
        request.write(url_request.body)
        writer.dispatch(request, self.__write_request_cb, [id, conn])

    def __write_request_cb(self, id, conn):
        reader = HTTPClientReader(conn)
        reader.get_full_response(self.__end_body_cb, [id])
#        print "asking for response"

    def __end_body_cb(self, response, id):
#        print "getting response"
#        print response.status_line
#        print response.headers
#        print response.body
        self.requests[id].success(response)

    def __parse_url(self, url):
        """ >>> __hostname_from_url("www.google.com/hello/world?q=yo")
            /, "www.google.com", 80
        """
        if url.startswith('http://'):
            url = url[7:]
        parts = url.split("/", 1)
        if len(parts) == 1:
            path = "/"
        else:
            path = "/" + parts[1]

        parts = parts[0].split(":", 1)
        if len(parts) == 1:
            port = 80
        else:
            port = int(parts[1])
        hostname = parts[0]
        
        return path, hostname, port
        
class URLRequest(object):
    def __init__(self, id, path, host, port, method, headers, cb, cbargs, eb, ebargs, body, timeout=None):
        self.id = id
        self.cb = cb
        self.path = path
        self.host = host
        self.port = port
        self.method = method
        self.headers = headers
        self.cb = cb
        self.cbargs = cbargs
        self.eb = eb
        self.ebargs = ebargs
        self.body = body
        self.timeout = event.event(self.failure)
        if timeout:
            self.timeout.add(timeout)
        self.failed = False
        
    def success(self, response):
        if not self.failed:
            self.timeout.delete()
            if self.cb:
                args = []
                if self.cbargs:
                    args = self.cbargs
                self.cb(response, *args)
        
    def failure(self, *args, **kwargs):
        self.failed = True
        if self.eb:
            self.eb(*self.ebargs)



from dez.network import WebSocketDaemon

def new_conn(conn):
    conn.write('you are connected!')
    def recv(frame):
        conn.write("ECHO: %s"%(frame))
    conn.set_cb(recv)

def log(msg):
    print "* " + msg

def main(domain, port):
    log("starting WebSocket Test")
    log("a sample client is provided in the 'dez/samples/html' directory")
    server = WebSocketDaemon(domain, port, new_conn, report_cb=log)
    server.start()


import event, os, struct, ctypes, ctypes.util

class INotify(object):
    def __init__(self, cb):
        self.__cb = cb
        self.__struct_size = 16
        self.__files = {}
        self.__clib = ctypes.cdll.LoadLibrary(ctypes.util.find_library('libc'))
        self.__fd = self.__clib.inotify_init()
        if self.__fd < 0:
            raise Exception, "INotify: bad file descriptor! Make sure your kernel supports inotify."
        event.read(self.__fd, self.__read_ready)

    def __read_ready(self):
        buff = os.read(self.__fd, 64)
        buff_index = 0
        while buff_index < len(buff):
            s = struct.unpack('iIII', buff[buff_index:buff_index+self.__struct_size])
            event.timeout(.1,self.__cb_handler,s)
            buff_index += self.__struct_size
        return True

    def __cb_handler(self, s):
        wd = s[0]
        if wd not in self.__files:
            return
        path = self.__files[wd]
        self.__cb(path)
        if s[1] == 2048:
            del self.__files[wd]
            self.add_path(path)

    def add_path(self, path):
        wd = self.__clib.inotify_add_watch(self.__fd, path, 4038)
        self.__files[wd] = path


class FakeLogger(object):
    def debug(self, *args, **kwargs):
        pass
    info = debug
    access = debug
    warn = debug
    error = debug
    
logger = FakeLogger()

def default_get_logger(name):
    return logger



import rel, time, socket
from optparse import OptionParser
from dez.http.client import HTTPClient
from dez.http.errors import HTTPProtocolError

class LoadTester(object):
    def __init__(self, host, port, path, number, concurrency):
        self.url = "http://"+host+":"+str(port)+path
        self.number = number
        self.concurrency = concurrency
        self.responses = 0
        print "\nInitializing Load Tester"
        display(" server url: %s"%self.url)
        display("     number: %s"%self.number)
        display("concurrency: %s"%self.concurrency)
        print "\nBuilding Connection Pool"
        self.client = HTTPClient()
        self.client.client.start_connections(host, port, self.concurrency, self.connections_open, max_conn=concurrency)
        self.t_start = time.time()

    def connections_open(self):
        self.t_connection = self.t_request = time.time()
        display("pool ready\n\nRunning Test Load")
        display("%s connections opened in %s ms"%(self.concurrency, ms(self.t_connection, self.t_start)))
        display("-")
        for i in range(self.number):
            self.client.get_url(self.url, cb=self.response_cb)

    def response_cb(self, response):
        self.responses += 1
        if self.responses == self.number:
            now = time.time()
            display("%s responses: %s ms"%(self.responses, ms(now, self.t_request)))
            display("\nRequests Per Second")
            display("%s requests handled in %s ms"%(self.number, ms(now, self.t_connection)))
            display("%s requests per second (without connection time)"%int(self.number / (now - self.t_connection)))
            display("%s requests per second (with connection time)"%int(self.number / (now - self.t_start)))
            rel.abort()
        elif not self.responses % 100:
            now = time.time()
            display("%s responses: %s ms"%(self.responses, ms(now, self.t_request)))
            self.t_request = now

def ms(bigger, smaller):
    return int(1000*(bigger - smaller))

def display(msg):
    print "   ",msg

def error(m1, m2):
    print '\n%s\n%s\n\ntry this: "dbench HOSTNAME PORT NUMBER CONCURRENCY"\nor "dbench -h" for help\n'%(m1,m2)

def main():
    parser = OptionParser("dbench HOSTNAME PORT NUMBER CONCURRENCY")
    parser.add_option("-p", "--path", dest="path", default="/", help="path -> http://[DOMAIN]:[PORT][PATH]")
    parser.add_option("-e", "--event", dest="event", default="epoll", help="change event delivery system (options: pyevent, epoll, poll, select) default: epoll")
    ops, args = parser.parse_args()
    if len(args) < 4:
        return error("insufficient arguments specified", "dbench requires 4 arguments")
    hostname = args[0]
    try:
        port = int(args[1])
        number = int(args[2])
        concurrency = int(args[3])
    except:
        return error("invalid argument","PORT, NUMBER, and CONCURRENCY must all be integers")
    print "\nLoading Event Listener"
    display(" requesting: %s"%ops.event)
    e = rel.initialize([ops.event])
    if e != ops.event:
        display("failed to load %s!"%ops.event)
    display("     loaded: %s"%e)
    print "\nTesting Server"
    test_sock = socket.socket()
    try:
        test_sock.connect((hostname, port))
        test_sock.close()
    except:
        return display("no server at %s:%s!\n\ngoodbye\n"%(hostname, port))
    display("valid server")
    def abort(msg):
        print ""
        print msg
        rel.abort()
    rel.signal(2, abort, "Test aborted by user")
    rel.timeout(30, abort, "Test aborted after 30 seconds")
    LoadTester(hostname, port, ops.path, number, concurrency)
    try:
        rel.dispatch()
    except HTTPProtocolError:
        print "\nerror communicating with server:"
        print "http protocol violation"
    finally:
        print "\ngoodbye\n"
        rel.abort()


from dez.network import SocketClient, SocketDaemon, Connection
import sys

def main(**kwargs):
    server = SocketDaemon(kwargs['domain'], kwargs['port'], cb=TestCloseChunked)
    server.start()


class TestCloseChunked(object):
    def __init__(self, conn):
        self.conn = conn
        self.conn.set_rmode_close_chunked(self.data_received)

    def data_received(self, data):
        print data.replace("\r\n", "\\r\\n\n")


class TestClose(object):
    def __init__(self, conn):
        self.conn = conn
        self.conn.set_rmode_close(self.data_received)

    def data_received(self, data):
        print data.replace("\r\n", "\\r\\n\n")


class TestSizeChunked(object):
    def __init__(self, conn):
        self.conn = conn
        self.conn.set_rmode_size_chunked(15, self.data_received)

    def data_received(self, data):
        print data.replace("\r\n", "\\r\\n\n")

class TestSize(object):
    def __init__(self, conn):
        self.conn = conn
        self.conn.set_rmode_size(15, self.data_received)

    def data_received(self, data):
        print data.replace("\r\n", "\\r\\n\n")


class TestDelimiter(object):
    def __init__(self, conn):
        self.conn = conn
        self.conn.set_rmode_delimiter('\r\n', self.data_received)

    def data_received(self, data):
        print data

#    def close(self):
#        self.conn.close()




import threading
import Queue

class WSGIThreadPool(object):
  
#    insert_stmt = stats.insert()
    
    def __init__(self, num_threads=5, ):
        self.num_threads = num_threads
        self.work_queue = Queue.Queue()
        self.threads = []
        for i in range(self.num_threads):
            self.threads.append(threading.Thread(target=self.run))
            
    def start(self):
        for i, thread in enumerate(self.threads):
            print "Starting WSGI thread", thread.getName()
            thread.start()

    def dispatch(self, response):
        self.work_queue.put(response)
        
    def run(self):
        name = threading.currentThread().getName()
        while True:
            try:
                response = self.work_queue.get()
                if not response:
                    break
                response.dispatch()
            except Exception, e:
                import sys, traceback
                print "error:", e
                print "todo: fix exception handling in threads"
                print traceback.print_tb(sys.exc_info()[2])
        print "Shutting down thread", name
                
    def stop(self):
        for t in self.threads:
            self.work_queue.put(None)
        # We aren't cleaning up our threadpool. For now, we're making
        # the assumption that if stop has been called it's because we
        # caught ^C (or something) and so we aren't worrying about
        # leaving stuff around.



from dez.network import SocketDaemon, SimpleClient
from datetime import datetime

class ReverseProxyConnection(object):
    def __init__(self, conn, h1, p1, h2, p2, logger, start_data):
        self.front_conn = conn
        self.front_host = h1
        self.front_port = p1
        self.back_host = h2
        self.back_port = p2
        self.logger = logger
        self.log("Initializing connection")
        SimpleClient().connect(h2, p2, self.onConnect, [start_data])

    def log(self, msg):
        self.logger("%s:%s -> %s:%s > %s"%(self.front_host, self.front_port, self.back_host, self.back_port, msg))

    def onConnect(self, conn, start_data):
        self.log("Connection established")
        self.back_conn = conn
        self.front_conn.set_close_cb(self.onClose, [self.back_conn])
        self.back_conn.set_close_cb(self.onClose, [self.front_conn])
        self.front_conn.set_rmode_close_chunked(self.back_conn.write)
        self.back_conn.set_rmode_close_chunked(self.front_conn.write)
        self.back_conn.write(start_data)

    def onClose(self, conn):
        self.log("Connection closed")
        self.front_conn.set_close_cb(None)
        self.back_conn.set_close_cb(None)
        self.front_conn.halt_read()
        self.back_conn.halt_read()
        self.front_conn = None
        self.back_conn = None
        conn.soft_close()

BIG_FILES = ["mp3", "png", "jpg", "jpeg",
    "gif", "pdf", "mov", "zip"] # more?

class ReverseProxy(object):
    def __init__(self, port, verbose):
        self.port = port
        self.default_address = None
        self.verbose = verbose
        self.domains = {}
        self.daemon = SocketDaemon('', port, self.new_connection)

    def log(self, data):
        if self.verbose:
            print "[%s] %s"%(datetime.now(), data)

    def new_connection(self, conn):
        conn.set_rmode_delimiter('\r\n\r\n', self.route_connection, [conn])

    def _302(self, conn, domain, path): # from hostTrick
        conn.write("HTTP/1.1 302 Found\r\nLocation: http://%s%s\r\n\r\n"%(domain, path))
        conn.soft_close()

    def domain2hostport(self, domain):
        if domain in self.domains:
            return self.domains[domain]
        if self.default_address:
            return self.default_address
        return None, None

    def cantroute(self, domain):
        msg = "unable to route hostname: %s"%(domain,)
        self.log(msg)
        conn.close(msg)

    def route_connection(self, data, conn):
        conn.halt_read()
        domain = None
        path = None
        should302 = False
        for line in data.split('\r\n'):
            if line.startswith("GET"):
                path = line.split(" ")[1]
                if "." in path and path.rsplit(".")[1] in BIG_FILES:
                    should302 = True
            elif line.startswith('Host: '):
                domain = line[6:]
                if ":" in domain:
                    domain = domain.split(":")[0]
                break
        if not domain:
            return conn.close('no host header')
        self.dispatch(data+'\r\n\r\n', conn, domain, should302, path)

    def dispatch(self, data, conn, domain, should302=False, path=None):
        host, port = self.domain2hostport(domain)
        if not host:
            return self.cantroute(domain)
        if should302:
            self._302(conn, "%s:%s"%(host, port), path)
        else:
            ReverseProxyConnection(conn, domain, self.port, host, port, self.log, data)

    def register_default(self, host, port):
        self.default_address = (host, port)

    def register_domain(self, domain, host, port):
        self.domains[domain] = (host, port)

    def start(self):
        self.daemon.start()

def error(msg):
    print "error:",msg
    import sys
    sys.exit(0)

def startreverseproxy():
    import os, optparse
    parser = optparse.OptionParser('dez_reverse_proxy [CONFIG]')
    parser.add_option("-v", "--verbose", action="store_true", dest="verbose", default=False, help="log proxy activity")
    parser.add_option("-p", "--port", dest="port", default="80", help="public-facing port (default: 80)")
    options, arguments = parser.parse_args()
    try:
        options.port = int(options.port)
    except:
        error('invalid port specified -- int required')
    if len(arguments) < 1:
        error("no config specified")
    config = arguments[0]
    if not os.path.isfile(config):
        error('no valid config - "%s" not found'%config)
    f = open(config)
    lines = f.readlines()
    f.close()
    try:
        controller = ReverseProxy(options.port, options.verbose)
    except:
        error('could not start server! try running as root!')
    for line in lines:
        line = line.split("#")[0]
        try:
            domain, back_addr = line.split('->')
            domain = domain.strip()
            host, port = back_addr.split(':')
            host = host.strip()
            port = int(port)
        except:
            error('could not parse config. expected "incoming_hostname -> forwarding_address_hostname:forwarding_address_port". failed on line: "%s"'%line)
        if domain == "*":
            print "Setting default forwarding address to %s:%s"%(host, port)
            controller.register_default(host, port)
        else:
            print "Mapping %s to %s:%s"%(domain, host, port)
            controller.register_domain(domain, host, port)
    print "Starting reverse proxy router on port %s"%(options.port)
    controller.start()



from dez.http.application import HTTPApplication

def main(**kwargs):
    a = HTTPApplication("", kwargs['port'])
    a.add_proxy_rule('/', kwargs['domain'], 80)
    a.add_static_rule('/static/', '.')
    a.start()


import event
from dez import io
from dez.buffer import Buffer
from dez.logging import default_get_logger
from dez.http.server.router import Router
from dez.http.server.response import RawHTTPResponse, HTTPResponse
from dez.http.server.request import HTTPRequest

class HTTPDaemon(object):
    def __init__(self, host, port, get_logger=None):
        if not get_logger:
            get_logger = default_get_logger
        self.log = get_logger("HTTPDaemon")
        self.get_logger = get_logger
        self.host = host
        self.port = port
        self.log.info("Listening on %s:%s" % (host, port))
        self.sock = io.server_socket(self.port)
        self.listen = event.read(self.sock, self.accept_connection, None, self.sock, None)
        self.router = Router(self.default_cb, [])

    def register_prefix(self, prefix, cb, args=[]):
        self.router.register_prefix(prefix, cb, args)

    def default_404_cb(self, request):
        r = HTTPResponse(request)
        r.status = "404 Not Found"
        r.write("The requested document %s was not found" % request.url)
        r.dispatch()

    def default_cb(self, request):
        return self.default_404_cb(request)

    def accept_connection(self, ev, sock, event_type, *arg):
        sock, addr = sock.accept()
        HTTPConnection(sock, addr, self.router, self.get_logger)
        return True

class HTTPConnection(object):
    id = 0
    def __init__(self, sock, addr, router, get_logger):
        self.log = get_logger("HTTPConnection")
        self.get_logger = get_logger
        HTTPConnection.id += 1
        self.id = HTTPConnection.id
        self.sock = sock
        self.addr, self.local_port = addr
        self.router = router
        self.response_queue = []
        self.current_cb = None
        self.current_args = None
        self.current_eb = None
        self.current_ebargs = None
        self.wevent = None
        self.revent = None
        self.__close_cb = None
        self.start_request()

    def set_close_cb(self, cb, args):
        self.__close_cb = (cb, args)

    def start_request(self):
        if self.wevent:
            self.wevent.delete()
            self.wevent = None
        if self.revent:
            self.revent.delete()
            self.revent = None
        self.revent = event.read(self.sock, self.read_ready)
        self.request = HTTPRequest(self)
        self.write_buffer = Buffer(mode='consume')
        self.buffer = Buffer()
        self.state = "read"

    def close(self, reason=""):
        if self.__close_cb:
            cb, args = self.__close_cb
            self.__close_cb = None
            cb(*args)
        if self.revent:
            self.revent.delete()
            self.revent = None
        if self.wevent:
            self.wevent.delete()
            self.wevent = None
        self.sock.close()
        if self.current_eb:
            self.current_eb(*self.current_ebargs)
            self.current_eb = None
            self.current_ebargs = None
        while self.response_queue:
            tmp = self.response_queue.pop(0)
            data, self.current_cb, self.current_args, self.current_eb, self.current_ebargs = tmp
            if self.current_eb:
                self.current_eb(*self.current_ebargs)
            self.current_eb = None
            self.current_ebargs = None

    def read_ready(self):
        try:
            data = self.sock.recv(io.BUFFER_SIZE)
            if not data:
                self.close()
                return None
            return self.read(data)
        except io.socket.error:
            self.close()
            return None

    def read_body(self):
        self.revent = event.read(self.sock, self.read_ready)
        self.request.process()

    def read(self, data):
        if self.state != "read":
            self.log.error("Invalid additional data: %s" % data)
            self.close()
        self.buffer += data
        self.request.process()
        if self.request.state == "waiting":
            self.revent.delete()
            self.revent = None
            dispatch_cb, args = self.router(self.request.url)
            dispatch_cb(self.request, *args)
            return None
        elif self.request.state == "completed":
            self.revent.delete()
            self.revent = None
            return None
        return True

    def write(self, data, cb, args,eb=None,ebargs=[]):
        self.response_queue.append((data, cb, args, eb, ebargs))
        if not self.wevent:
            self.wevent = event.write(self.sock, self.write_ready)

    def write_ready(self):
        if self.write_buffer.empty():
            if self.current_cb:
                cb = self.current_cb
                args = self.current_args
                cb(*args)
                self.current_cb = None
            if not self.response_queue:
                self.current_cb = None
                self.current_response = None
                self.wevent = None
                return None
            data, self.current_cb, self.current_args, self.current_eb, self.current_ebargs = self.response_queue.pop(0)
            self.write_buffer.reset(data)
            # call conn.write("", cb) to signify request complete
            if data == "":
                self.wevent = None
                self.current_cb(*self.current_args)
                self.current_cb = None
                self.current_args = None
                self.current_eb = None
                self.current_ebargs = None
                return None
        try:
            bsent = self.sock.send(self.write_buffer.get_value())
            self.write_buffer.move(bsent)
            self.log.debug('write_buffer: return True')
            return True
        except io.socket.error, msg:
            self.log.debug('io.socket.error: %s' % msg)
            self.close(reason=str(msg))
            return None


class Router(object):
    def __init__(self, default_cb, default_args=[]):
        self.default_cb = default_cb
        self.default_args = default_args
        self.prefixes = []

    def register_prefix(self, prefix, cb, args):
        self.prefixes.append((prefix, cb, args))
        self.prefixes.sort(self.pref_order)

    def pref_order(self, b, a):
        return cmp(len(a[0]),len(b[0]))

    def __call__(self, url):
        for prefix, cb, args in self.prefixes:
            if url.startswith(prefix):
                return cb, args
        return self.default_cb, self.default_args



class Frame(object):
    def __init__(self, conn, buff):
        self.conn = conn
        self.request = buff.data
        self.problem = None
        self.action = None
        self.headers = {}
        try:
            problem_at = "action"
            i = buff.find("\r\n")
            self.action = buff.part(0,i).strip()
            buff.move(i+2)
            problem_at = "headers"
            i = buff.find("\r\n\r\n")
            self.headers = {}
            for line in buff.part(0,i).split('\r\n'):
                key, val = line.split(':',1)
                key, val = key.strip(), val.strip()
                problem_at = "headers: %s"%key
                if key == "recipient":
                    if "recipients" not in self.headers:
                        self.headers["recipients"] = []
                    self.headers["recipients"].append(val)
                else:
                    self.headers[key] = val
            buff.move(i+4)
            problem_at = "body"
            self.body = buff.data
        except:
            raise
            self.problem = 'parse error in %s'%problem_at

    def error(self, msg, cb=None, cbarg=()):
        if msg == self.problem:
            h = {"code":"1","reason":"error parsing frame"}
            msg += "\r\n---frame---\r\n"+self.request+"\r\n-----------"
        elif "id" not in self.headers:
            h = {"code":"2","reason":"no id in headers"}
            msg += "\r\n---frame---\r\n"+self.request+"\r\n-----------"
        else:
            h = {"code":"3","error":"","id":self.headers["id"]}
        self.respond("ERROR", h, msg, cb, cbarg)

    def received(self, cb=None, cbarg=()):
        if self.headers["response"] == "receipt":
            self.respond("RECEIPT", cb=cb, cbarg=cbarg)

    def respond(self, action, headers=None, body="", cb=None, cbarg=()):
        if not headers:
            headers = {"id":self.request_id}
        self.conn.write(action, headers, body, cb, cbarg)

class SendFrame(Frame):
    def success(self, succ, cb=None, cbarg=()):
        self.conn.callback("success",{"id":self.request_id,"recipients":succ},cb,cbarg)

    def failure(self, fail, cb=None, cbarg=()):
        self.conn.callback("failure",{"id":self.request_id,"recipients":fail},cb,cbarg)



from dez.http.server import HTTPResponse, HTTPVariableResponse
from dez.http.cache import NaiveCache, INotifyCache
from dez import io
import os

class StaticHandler(object):
    def __init__(self, server_name):
        self.server_name = server_name
        try:
            self.cache = INotifyCache()
            #print "static cache: INotifyCache"
        except:
            self.cache = NaiveCache()
            #print "static cache: NaiveCache"

    def __call__(self, req, prefix, directory):
        path = os.path.join(directory, req.url[len(prefix):])
        if os.path.isdir(path):
            url = req.url
            if url.endswith('/'):
                url = url[:-1]
            response = HTTPResponse(req)
            response.headers['Server'] = self.server_name
            response.write('<b>%s</b><br><br>'%url)
            response.write("<a href=%s>..</a><br>"%os.path.split(url)[0])
            for child in os.listdir(path):
                response.write("<a href=%s/%s>%s</a><br>"%(url,child,child))
            return response.dispatch()
        self.cache.get(req, path, self.__write, self.__stream, self.__404)

    def __write(self, req, path):
        response = HTTPResponse(req)
        response.headers['Server'] = self.server_name
        response.headers['Content-type'] = self.cache.get_type(path)
        response.write(self.cache.get_content(path))
        response.dispatch()

    def __stream(self, req, path):
        response = HTTPVariableResponse(req)
        response.headers['Server'] = self.server_name
        response.headers['Content-type'] = self.cache.get_type(path)
        self.__write_file(response, open(path), path)

    def __404(self, req):
        response = HTTPResponse(req)
        response.headers['Server'] = self.server_name
        response.status = '404 Not Found'
        response.write("<b>404</b><br>Requested resource \"<i>%s</i>\" not found" % req.url)
        response.dispatch()

    def __write_file(self, response, openfile, path):
        data = openfile.read(io.BUFFER_SIZE)
        if data == "":
            return response.end()
        self.cache.add_content(path, data)
        response.write(data, self.__write_file, [response, openfile, path])


from dez.stomp.server.server import STOMPServer, STOMPConnection
from orbited.op.message import OrbitMessage
import random,event

def _key():
    return ''.join([random.choice('0123456789ABCDEF') for i in range(8)])

def _deliver(d):
    event.timeout(random.random(),random.choice([d.success,d.failure]))

class TestDaemon(object):
    def request_cb(self, frame, *args):
        print 'request_cb called'
        print 'args',args
        print 'request:'
        print '----'
        print frame.request
        print '----'
        if frame.action == "CONNECT":
            frame.connected(_key())
        elif frame.action == "DISCONNECT":
            frame.close()
        elif frame.action == "SEND":
            dest = frame.headers["destination"]
            msg = OrbitMessage([dest], frame.body, self.msg_outcome, [frame])
            _deliver(msg.single_recipient_message(dest))

    def msg_outcome(self, message, frame):
        if message.success_recipients:
            frame.success(message.success_recipients)
        if message.failure_recipients:
            frame.failure(message.failure_recipients)

    def close_cb(self, conn, *args):
        print 'connection %s closed' % conn
        print 'args:',args

    def connect_cb(self, conn):
        conn.set_request_cb(self.request_cb, ['print', 'this', 'out'])
        conn.set_close_cb(self.close_cb, ['conn', 'closed'])

def main(**kwargs):
    test = TestDaemon()
    server = STOMPServer(kwargs['domain'], kwargs['port'])
    server.set_connect_cb(test.connect_cb)
    server.start()




import socket
LQUEUE_SIZE = 5
BUFFER_SIZE = 4096

def server_socket(port):
    ''' Return a listening socket bound to the given interface and port. '''
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    sock.setblocking(0)
    sock.bind(('', port))
    sock.listen(LQUEUE_SIZE)
    return sock

def client_socket(addr, port):
    sock = socket.socket()
    sock.setblocking(False)
    try:
        sock.connect_ex((addr, port))
    except socket.error:
        # this seems to happen when there are
        # > 1016 connections, for some reason.
        # we need to get to the bottom of this
        raise SocketError, "the python socket cannot open another connection"
    return sock

class SocketError(Exception):
    pass



from dez.network.client import SocketClient
from dez.http.client.response import HTTPClientReader

client = SocketClient()

def proxy(request, dest_host, dest_port):
    client.get_connection(dest_host, dest_port, connection_cb, [request])

def connection_cb(conn, request):
    conn.write(request.action + '\r\n')
    for (key, val) in request.headers.items():
        conn.write('%s: %s\r\n' % (key, val))
    conn.write('\r\n')
    request.read_body_stream(body_cb, [conn, request])

def body_cb(data, conn, request):
    if not data:
        if request.headers.get('x-requested-with', None) == 'XMLHttpRequest':
            return conn.set_rmode_close(xhr_cb, [request])
        reader = HTTPClientReader(conn)
        reader.get_headers_only(response_headers_cb, [reader, request])
    else:
        conn.write(data) # is this right?

def xhr_cb(data, request):
    request.write(data)
    request.close()

def response_headers_cb(response, reader, request):
    request.write(response.status_line + '\r\n')
    for (key, val) in response.case_match_headers.items():
        request.write('%s: %s\r\n' % (val, response.headers[key]))
    request.write('\r\n')
    reader.get_body_stream(response_body_cb, [ request])

def response_body_cb(response, request):
    data = response.body.get_value()
    if 'content-length' not in response.headers:
        request.write("%X\r\n%s\r\n"%(len(data),data))
    if not len(data):
        return request.end()
    if 'content-length' in response.headers:
        request.write(data)
    response.body.reset()


from dez.network.client import SocketClient
import event
def main(**kwargs):
    port = kwargs['port']
    domain = kwargs['domain']
    x = SocketClient()
    x.get_connection(domain, port, get_connection_cb, [ "hello10"])
    x.get_connection(domain, port, get_connection_cb, [ "hello9" ])
    x.get_connection(domain, port, get_connection_cb, [ "hello8" ])
    x.get_connection(domain, port, get_connection_cb, [ "hello7" ])
    x.get_connection(domain, port, get_connection_cb, [ "hello6" ])
    x.get_connection(domain, port, get_connection_cb, [ "hello5" ])
    x.get_connection(domain, port, get_connection_cb, [ "hello4" ])
    x.get_connection(domain, port, get_connection_cb, [ "hello3" ])
    x.get_connection(domain, port, get_connection_cb, [ "hello2" ])
    x.get_connection(domain, port, get_connection_cb, [ "hello1" ])
    event.signal(2, event.abort)
    event.dispatch()

def get_connection_cb(conn, payload):
    conn.write(payload, hello_world_cb, [conn])

def hello_world_cb(conn):
    conn.release()
    print 'releasing'
    



import re

from dez.http.server import HTTPDaemon, HTTPResponse, HTTPVariableResponse, WSGIResponse
from dez.http.static import StaticHandler
from dez.http.wsgithreadpool import WSGIThreadPool
from dez.http.proxy.proxy import proxy
from dez.http.errors import HTTPProtocolError

import event
import cgi

class HTTPApplication(object):
    """
        HTTPApplication depends on HTTPDaemon for its prefix ordering behavior.
        It issues cbs as per add_cb_rule in the same manner that HTTPDaemon
        does as well -- it simply passes a dez.http.server.request.HTTPRequest
        instance as the single argument to the callback. Its up to the user to
        construct the appropriate RawHTTPResponse or HTTPResponse as required.
    """

    def __init__(self, bind_address, port, get_logger=None, server_name="Dez"):
        """start listening on the given port (this doesn't include a call to
           event.dispatch)"""
        self.daemon = HTTPDaemon(bind_address, port, get_logger)
        self.host = bind_address
        self.port = port
        self.static_request = StaticHandler(server_name)
        self.wsgi_pool = None
        
    def start(self):
        """calls event.dispatch"""
        event.signal(2, self.stop)
        if self.wsgi_pool:
            self.wsgi_pool.start()
        event.dispatch()

    def stop(self):
        if self.wsgi_pool:
            self.wsgi_pool.stop()
        event.abort()
        
    def add_proxy_rule(self, prefix, dest_host, dest_port):
        """ Adds a prefix that will be proxied to the destination
            http_app.add_proxy_rule("/chat/", "127.0.0.1", 4700)
            url == "/chat/index.html"  # will be proxied
            url == "/hello/chat/index.html" # will not be proxied
        """
        self.daemon.register_prefix(prefix, self.__proxy, [dest_host, dest_port])
        
        
    def __proxy(self, req, host, port):
        return proxy(req, host, port)

    def add_static_rule(self, prefix, local_base_resource):
        """ Adds a prefix that will be served as a static directory or file
            http_app.add_static_rule("/static/", "/home/app/static")
            url == "/static/css/common.css" refers to /home/app/static/css/common.css

            http_app.add_static_rule("/", "/home/app/static/html/index.html")
            url == "/" refers to /home/app/static/html/index.html

            http_app.add_static_rule("/main.html", "/home/app/static/html/main.html")
            url == "/main.html" refers to /home/app/static/html/main.html
            url == "/main.html/foo" returns a 404
        """
        self.daemon.register_prefix(prefix, self.static_request, [prefix, local_base_resource])
#        self.static.register(prefix, local_base_resource)
#        raise Exception("NotImplemented"), "Static file serving not implemented"

    def add_cb_rule(self, prefix, cb, parsed=True):
        """ Adds a prefix that will issue a call to cb when a request with that
            url prefix is matched. This works the same as the HTTPDaemon's
            register_prefix function.

            def dispatch(req):
                req.write("HTTP/1.0 200 OK\r\n\r\nHello World")
                req.close()

            http_app.add_cb_rule("/helloworld", dispatch)
        """
        app_cb = cb
        if parsed:
            def format_wrapper(req):
                req = ParsedHTTPRequest(req, cb)
            app_cb = format_wrapper
        self.daemon.register_prefix(prefix, app_cb)

    def add_wsgi_rule(self, prefix, app):
        """ Adds a prefix that will execute (call) a wsgi compliant app function.
            Will provide the app with the appropriate environ and start_response
            function. 
        """
        if not self.wsgi_pool:
            self.wsgi_pool = WSGIThreadPool()
            
        script_name = prefix
        script_name_match = re.match(r"/(\S+?)\/?$", script_name)
        if script_name_match:
            script_name = "/" + script_name_match.group(1)
        
        if script_name != "/":
            try:
                from paste.deploy.config import PrefixMiddleware
                app = PrefixMiddleware(app, prefix=script_name)
            except ImportError:
                message = "could not import PrefixMiddleware from\n"
                message += "paste.deploy.config.\n"
                message += "You must install PasteDeploy before you can"
                message += " load a Pylons application.\n"
                raise ImportError(message)
        
        def wsgi_cb(req):
            r = WSGIResponse(req, app, self.host, self.port)
            self.wsgi_pool.dispatch(r)
#            r.dispatch()
        self.daemon.register_prefix(prefix, wsgi_cb)


class ParsedHTTPRequest(object):
    
    def __init__(self, req, cb):
        self.req = req
        self.cb = cb
        self.form = {}
        self.cookies = {}
        req.read_body(self.recv_body)
        
    def __getattr__(self, key):
        try:
            return object.__getattr__(self, key)
        except AttributeError:
          return getattr(self.req, key)
        
    def recv_body(self, body):
        self.body = body
        self.dispatch()  
      
    def dispatch(self):
        self.setup_url()
        self.setup_form()
        self.setup_cookies()
        self.cb(self)
        
    def setup_url(self):
        if self.req.method.lower() == "get" and "?" in self.req.url:
            self.url, self.qs = self.req.url.split('?', 1)
        elif self.req.method.lower() == "post":
            self.url, self.qs = self.req.url, self.body
        else:
            self.url, self.qs = self.req.url, ""
            
    def setup_form(self):
        try:
            for key, val in cgi.parse_qsl(self.qs):
                self.form[key] = val
        except ValueError:
            raise HTTPProtocolError, "Invalid querystring format"
        
    def setup_cookies(self):
        pass

def serve_wsgi_application(application, global_conf, **kwargs):
    """ Serve a WSGI application using a paste.deploy.config
    
        This is an adapter to allow dez to be used with the paster
        serve mechanism as interpreted through a *.ini file.
    """
    
    host = kwargs.pop("host", "127.0.0.1")
    port = int(kwargs.pop("port", "8888"))
    
    print "listening on %s:%s" % (host, port)
    httpd = HTTPApplication(host, port, **kwargs)
    httpd.add_wsgi_rule("/", application)
    httpd.start() 



import optparse
from base64 import b64encode
from hashlib import sha1
from datetime import datetime
from dez.buffer import Buffer
from dez.network.server import SocketDaemon
from dez.network.client import SimpleClient

GUID = "258EAFA5-E914-47DA-95CA-C5AB0DC85B11"

def key2accept(key):
    return b64encode(sha1(key + GUID).digest())

def parse_frame(buf):
    """
    Parse a WebSocket frame. If there is not a complete frame in the
    buffer, return without modifying the buffer.

    Adapted from:
    http://www.cs.rpi.edu/~goldsd/docs/spring2012-csci4220/websocket-py.txt
    """
    payload_start = 2

    # try to pull first two bytes
    if len(buf) < 3:
        return
    b = ord(buf[0])
    fin = b & 0x80      # 1st bit
    # next 3 bits reserved
    opcode = b & 0x0f   # low 4 bits
    b2 = ord(buf[1])
    mask = b2 & 0x80    # high bit of the second byte
    length = b2 & 0x7f    # low 7 bits of the second byte

    # check that enough bytes remain
    if len(buf) < payload_start + 4:
        return
    elif length == 126:
        length, = struct.unpack(">H", buf[2:4])
        payload_start += 2
    elif length == 127:
        length, = struct.unpack(">I", buf[2:6])
        payload_start += 4

    if mask:
        mask_bytes = [ord(b) for b in buf[payload_start:payload_start + 4]]
        payload_start += 4

    # is there a complete frame in the buffer?
    if len(buf) < payload_start + length:
        return

    # remove leading bytes, decode if necessary, dispatch
    payload = buf[payload_start:payload_start + length]
    buf.move(payload_start + length)

    # use xor and mask bytes to unmask data
    if mask:
        unmasked = [mask_bytes[i % 4] ^ ord(b)
            for b, i in zip(payload, range(len(payload)))]
        payload = "".join([chr(c) for c in unmasked])

    return payload

class WebSocketProxy(object):
    def __init__(self, myhostname, myport, targethostname, targetport, b64=False, verbose=False):
        self.verbose = verbose
        self.target = {'host':targethostname, 'port':targetport, 'b64':b64}
        self.proxy = WebSocketDaemon(myhostname, myport, self._new_conn, b64, self._report)

    def _report(self, data):
        if self.verbose:
            print "%s [WebSocketProxy] %s"%(datetime.now(), data)

    def _new_conn(self, conn):
        self._report("Connecting to TCP server")
        WebSocketProxyConnection(conn, self.target, self._report)

    def start(self):
        self._report("Proxy started")
        self.proxy.start()

class WebSocketProxyConnection(object):
    def __init__(self, client2ws, target, report_cb=lambda x:None):
        self.client2ws = client2ws
        self.report_cb = report_cb
        SimpleClient(target['b64']).connect(target['host'], target['port'], self._conn_server)

    def _conn_server(self, ws2server):
        self.report_cb("TCP connection ready")
        self.ws2server = ws2server
        self.ws2server.set_rmode_close_chunked(self.client2ws.write)
        self.client2ws.set_cb(self.ws2server.write)
        def do_nothing():pass
        def ws2server_close():
            self.report_cb("TCP server disconnected")
            self.report_cb("Closing client connection")
            self.client2ws.set_close_cb(do_nothing)
            self.client2ws.close()
        def client2ws_close():
            self.report_cb("Client disconnected")
            self.report_cb("Closing TCP connection")
            self.ws2server.set_close_cb(do_nothing)
            self.ws2server.close()
        self.ws2server.set_close_cb(ws2server_close)
        self.client2ws.set_close_cb(client2ws_close)
        self.report_cb("Connections linked")

class WebSocketDaemon(SocketDaemon):
    def __init__(self, hostname, port, cb, b64=False, report_cb=lambda x:None):
        SocketDaemon.__init__(self, hostname, port, cb, b64)
        real_cb = self.cb
        def handshake_cb(conn):
            report_cb("Handshake initiated")
            WebSocketHandshake(conn, hostname, port, real_cb, report_cb)
        self.cb = handshake_cb

class WebSocketHandshake(object):
    def __init__(self, conn, hostname, port, cb, report_cb=lambda x:None):
        self.hostname = hostname
        self.port = port
        self.cb = cb
        self.report_cb = report_cb
        self.conn = conn
        self.conn.set_rmode_delimiter('\r\n', self._recv_action)

    def _handshake_error(self, data):
        self.report_cb(data)
        self.report_cb("Connection closed")
        self.conn.close()

    def _recv_action(self, data):
        self.report_cb("Processing action line")
        tokens = data.split(' ')
        if len(tokens) != 3:
            return self._handshake_error("Invalid action line")
        self.path = tokens[1]
        self.conn.set_rmode_delimiter('\r\n\r\n', self._recv_headers)

    def _recv_headers(self, data):
        self.report_cb("Processing headers")
        lines = data.split('\r\n')
        self.headers = {}
        for line in lines:
            header = line.split(': ', 1)
            if len(header) != 2:
                return self._handshake_error("Invalid headers")
            self.headers.__setitem__(*header)
        for required_header in ['Host', 'Origin', 'Sec-WebSocket-Key']:
            if required_header not in self.headers:
                return self._handshake_error("Missing header: %s"%(required_header,))
        response_headers = [
            "Upgrade: WebSocket",
            "Connection: Upgrade",
            "WebSocket-Origin: %s"%(self.headers['Origin'],),
            "WebSocket-Location: ws://%s:%s%s"%(self.hostname, self.port, self.path),
            "Sec-WebSocket-Accept: %s"%(key2accept(self.headers['Sec-WebSocket-Key']),)
        ]
        self.conn.write("HTTP/1.1 101 Web Socket Protocol Handshake\r\n%s\r\n\r\n"%("\r\n".join(response_headers),))
        self.report_cb("Handshake complete")
        self.cb(WebSocketConnection(self.conn, self.report_cb))

class WebSocketConnection(object):
    def __init__(self, conn, report_cb=lambda x:None):
        self.buff = Buffer()
        self.conn = conn
        self.conn.halt_read()
        self.report_cb = report_cb
        self.report_cb("Client connection ready")

    def _recv(self, data):
        self.report_cb('Data received:"%s"'%(data,))
        self.buff += data
        payload = parse_frame(self.buff)
        if payload:
            self.report_cb('Payload parsed:"%s"'%(payload,))
            self.cb(payload)

    def set_close_cb(self, cb):
        self.conn.set_close_cb(cb)

    def set_cb(self, cb):
        self.cb = cb
        self.conn.set_rmode_close_chunked(self._recv)

    def write(self, data):
        self.report_cb('Data sent:"%s"'%(data))
        self.conn.write(chr(0x81) + chr(len(data)) + data)

    def close(self):
        self.conn.close()

def startwebsocketproxy():
    parser = optparse.OptionParser('dez_websocket_proxy [DOMAIN] [PORT]')
    parser.add_option("-v", "--verbose", action="store_true", dest="verbose", default=False, help="verbose output")
    options, args = parser.parse_args()
    try:
        hostname, port = args
    except:
        print '\ndez_websocket_proxy is run with two arguments: the hostname and port of the server being proxied to. For example:\n\ndez_websocket_proxy mydomain.com 5555\n\nwill run a WebSocket server that listens for connections on port 81 and proxies them to a TCP server at mydomain.com:5555.'
        return
    try:
        port = int(port)
    except:
        print '\nThe second argument must be an integer. The command should look like this:\n\ndez_websocket_proxy mydomain.com 5555\n\nTry again!'
        return
    try:
        proxy = WebSocketProxy('localhost', 81, hostname, port, verbose=options.verbose)
    except:
        print '\nPermission denied to use port %s. Depending on how your system is set up, you may need root privileges to run the proxy.'%(port)
        return
    print 'running WebSocket server on port 81'
    print 'proxying to %s:%s'%(hostname, port)
    proxy.start()


class STOMPValidator(object):
    def __init__(self):
        self.actions = {
            "CONNECT": [],
            "SEND": ['destination'],
            "SUBSCRIBE": ['destination'],
            "UNSUBSCRIBE": [],
            "BEGIN": ['transaction'],
            "COMMIT": ['transaction'],
            "ABORT": ['transaction'],
            "ACK": ['message-id'],
            "DISCONNECT": [],
        }

    def _error(self, req):
        estring = "ACTION:"+req.action
        estring += "\r\nHEADERS:"+str(req.headers)
        return estring

    def __call__(self, req):
        if req.problem:
            req.error(req.problem,self._error(req))
            return False
        if req.action not in self.actions:
            req.error("invalid action:%s"%req.action,self._error(req))
            return False
        for header in self.actions[req.action]:
            if header not in req.headers:
                req.error("'%s' header missing from '%s' action"%(header,req.action),self._error(req))
                return False
        if req.action == "CONNECT" and "receipt" in req.headers:
            req.error("'receipt' header in CONNECT - not allowed",self._error(req))
            return False
        if req.action == "UNSUBSCRIBE" and "destination" not in req.headers and "id" not in req.headers:
            req.error("'destination' header and 'id' header not in UNSUBSCRIBE (you need at least one)",self._error(req))
            return False
        return True


from dez.buffer import Buffer
from dez.http.errors import *

class HTTPClientReader(object):
    def __init__(self, conn):
        self.conn = conn
        self.cb_info = (None, None)
        self.mode = "wait"
        self.response = HTTPClientResponse()
        self.helper = HTTPProtocolReader(conn, self.response)

    def get_full_response(self, cb, cbargs):
        self.cb_info = (cb, cbargs)
        self.helper.get_headers(self.__full_headers_end_cb, ())

    def __full_headers_end_cb(self):
#        print "__full_headers_end_cb"
        self.helper.get_body(self.__dispatch_cb, ())

    def get_headers_only(self, cb, cbargs):
        self.cb_info = (cb, cbargs)
        self.helper.get_headers(self.__dispatch_cb, ())

    def __dispatch_cb(self):
#        print "__dispatch_cb"
        cb, args = self.cb_info
        if cb:
            if args is None:
                args = []
            return cb(self.response, *args)

    def get_body(self, cb, cbargs):
        self.cb_info = (cb, cbargs)
        self.helper.get_body(self.__dispatch_cb, ())

    def get_body_stream(self, cb, cbargs):
        self.cb_info = (cb, cbargs)
        self.helper.get_body_stream(self.__dispatch_stream_cb, ())

    def __dispatch_stream_cb(self):
        cb, args = self.cb_info
        if cb:
            if args is None:
                args = []
            return cb(self.response, *args)

class HTTPClientResponse(object):
    def __init__(self):
        self.version_major = None
        self.version_minor = None
        self.status_line = None
        self.status_code = None
        self.status_msg = None
        self.headers = {}
        self.case_match_headers = {}
        self.content_length = None
        self.body = Buffer()
        self.completed = False

class HTTPProtocolReader(object):
    def __init__(self, conn, response):
        self.response = response
        self.conn = conn
        self.cb_info = (None, None)
#        self.body_mode = None
#        self.test_buff = ""

    def get_headers(self, cb, args):
#        print 'get_headers'
        self.cb_info = (cb, args)
#        return self.test_function("")############
        self.conn.set_rmode_delimiter('\r\n', self.__recv_status)

########
#    def test_function(self, data):
#        self.test_buff += data
#        print 'buff: "%s"'%self.test_buff
#        if self.test_buff.endswith("\r\n"):
#            self.__recv_status(self.test_buff[:-2])
#            self.test_buff = ""
#            return
#        self.conn.set_rmode_size(1, self.test_function)
#######

    def __recv_status(self, data):
#        print '__recv_status'
        r = self.response
        r.status_line = data
        try:
            r.protocol, status_code, r.status_msg = data.split(' ',2)
            r.status_code = int(status_code)
        except ValueError:
            raise HTTPProtocolError, "Invalid HTTP status line \"%s\"" % r.status_line
        #self.protocol = self.protocol.lower()
        url_scheme, r.version = r.protocol.split('/',1)
        major, minor = r.version.split('.', 1)
        r.version_major = int(major)
        r.version_minor = int(minor)
        self.conn.set_rmode_delimiter('\r\n', self.__recv_header)

    def __recv_header(self, data):
        if len(data) == 0:
            self.conn.halt_read()
            cb, args, = self.cb_info
            return cb(*args)
        key, val = data.split(': ')
        self.response.case_match_headers[key.lower()] = key
        self.response.headers[key.lower()] = val
        if key.lower() == 'content-length':
            self.response.content_length = int(val)
#            self.response.content_read = 0

    def get_body(self, cb, args):
        if self.response.content_length == 0:
            # return cb("", *args) # this line causes an error
            return cb(*args) # this should work
        self.cb_info = (cb, args)
        # TODO: add transfer-encoding chunked
        if self.response.content_length:
            return self.conn.set_rmode_size(self.response.content_length, self.__recv_body)
        if self.response.version_minor == 0:
            return self.conn.set_rmode_close(self.__recv_body)
        if self.response.headers.get('transfer-encoding', None) == 'chunked':
            return self.conn.set_rmode_delimiter('\r\n', self.__recv_chunk_head)

        raise HTTPProtocolError, "HTTP/1.1 must set content-length or transfer-encoding: chunked"

    def get_body_stream(self, cb, args):
        if self.response.content_length == 0:
            return cb("", *args)
        self.cb_info = (cb, args)
        if self.response.content_length:
            return self.conn.set_rmode_size_chunked(self.response.content_length, self.__recv_body_stream)
        if self.response.version_minor == 0:
            return self.conn.set_rmode_close_chunked(self.__recv_body_stream)
        if self.response.headers.get('transfer-encoding', None) == 'chunked':
            self.conn.set_rmode_delimiter('\r\n', self.__recv_chunk_head, [True])
#    self.conn.set_rmode_delimiter(    

    def __recv_body(self, data):
        # do something with self.response
        self.response.body += data
        self.conn.release()
        cb, args = self.cb_info
        return cb(*args)

    def __recv_chunk_head(self, data, stream=False):
        #stream = False
        i = data.find(';')
        if i == -1:
            size = data
            encodings = ""
        else:
            size = data[:i]
            encodings = data[i+1:]
        chunk_size = int(size, 16) + 2 # Account for trailing \r\n
#        print 'got', size, '(', chunk_size, ') head'
        if stream:
            self.conn.set_rmode_size(chunk_size, self.__recv_chunk_body_stream)
        else:
#            print 'set_rmode_size(', chunk_size, ', self.__recv_chunk_body)'
            self.conn.set_rmode_size(chunk_size, self.__recv_chunk_body)

    def __recv_chunk_body(self, data):
        data = data[:-2] # account for trailing \r\n
#        print 'got', len(data), 'length chunk'
        self.response.body += data
        if len(data) > 0:
            self.conn.set_rmode_delimiter('\r\n', self.__recv_chunk_head)
        else:
#            self.conn.release()
            self.__recv_body("")

    def __recv_chunk_body_stream(self, data):
        data = data[:-2] # account for trailing \r\n
        if len(data) > 0:
            self.conn.set_rmode_delimiter('\r\n', self.__recv_chunk_head, [True])
        self.__recv_body_stream(data)

    def __recv_body_stream(self, data):
        cb, args = self.cb_info
        self.response.body += data
#        if self.response.content_read != None:
#            self.response.content_read += len(data)
        if not len(data):
            #print 'release'
            #self.conn.close()
            self.conn.release()
        return cb(*args)


import mimetypes, os
from dez.http.inotify import INotify

class BasicCache(object):
    def __init__(self):
        self.cache = {}

    def _mimetype(self, url):
        mimetype = mimetypes.guess_type(url)[0]
        if not mimetype:
            mimetype = "application/octet-stream"
        return mimetype

    def get_type(self, path):
        return self.cache[path]['type']

    def get_content(self, path):
        return self.cache[path]['content']

    def add_content(self, path, data):
        self.cache[path]['content'] += data

    def get(self, req, path, write_back, stream_back, err_back):
        if self._is_current(path):
            return write_back(req, path)
        if os.path.isfile(path):
            self._new_path(path, req.url)
            return stream_back(req, path)
        err_back(req)

class NaiveCache(BasicCache):
    def _is_current(self, path):
        return path in self.cache and self.cache[path]['mtime'] == os.path.getmtime(path)

    def _new_path(self, path, url):
        self.cache[path] = {'mtime':os.path.getmtime(path),'type':self._mimetype(url),'content':''}

class INotifyCache(BasicCache):
    def __init__(self):
        self.cache = {}
        self.inotify = INotify(self.__update)

    def __update(self, path):
        f = open(path,'r')
        self.cache[path]['content'] = f.read()
        f.close()

    def _is_current(self, path):
        return path in self.cache

    def _new_path(self, path, url):
        self.cache[path] = {'type':self._mimetype(url),'content':''}
        self.inotify.add_path(path)


from dez.op.server.connection import Callback_Handler
import event

def main(**kwargs):
    c = Callback_Handler(None)
    domain = kwargs['domain']
    port = kwargs['port']
    c.set_url("success","http://"+domain+":"+str(port)+"/")
    c.set_url("failure","http://"+domain+":"+str(port)+"/")
    c.dispatch("success",{"key1":"value1","key2":"value2"})
    c.dispatch("failure",{"fkey":"fval","recipients":["r1","r2","r3","r4","r5"]})
    event.timeout(2,event.abort)
    event.dispatch()



import event
from dez.http.errors import HTTPProtocolError

class HTTPRequest(object):
    
    def __init__(self, conn):
        self.conn = conn
        self.log = conn.get_logger('HTTPRequest')
        
        self.state = 'action'
        self.headers = {}
        self.case_match_headers = {}
        self.headers_complete = False
        self.complete = False
        self.write_ended = False
        self.send_close = False
        self.body = None
        self.body_cb = None
        self.body_stream_cb = None
        self.remaining_content = 0
        self.write_queue_size = 0
        self.pending_actions = []
            
    def process(self):
        return getattr(self, 'state_%s' % self.state)()        
    
    def set_close_cb(self, cb, args):
        self.conn.set_close_cb(cb, args)
    
    def state_action(self):
        if '\r\n' not in self.conn.buffer:
            return False
        i = self.conn.buffer.find('\r\n')
        self.action = self.conn.buffer.part(0,i)
        try:
            self.method, self.url, self.protocol = self.conn.buffer.part(0,i).split(' ',2)
        except ValueError:
            raise HTTPProtocolError, "Invalid HTTP status line"
        #self.protocol = self.protocol.lower()
        url_scheme, version = self.protocol.split('/',1)
        major, minor = version.split('.', 1)
        self.version_major = int(major)
        self.version_minor = int(minor)
        self.url_scheme = url_scheme.lower()
        self.conn.buffer.move(i+2)
        
        
        self.state = 'headers'
        return self.state_headers()
    
    def state_headers(self):
        while True:
            index = self.conn.buffer.find('\r\n')
            if index == -1:
                return False
            if index == 0:
                self.conn.buffer.move(2)
                self.state='headers_completed'
                return self.state_headers_completed()
            try:
                key, value = self.conn.buffer.part(0, index).split(': ')
            except ValueError:
                raise HTTPProtocolError, "Invalid HTTP header format"
            self.headers[key.lower()] = value
            self.case_match_headers[key.lower()] = key

            self.conn.buffer.move(index+2)
            
            
    def state_headers_completed(self):
        self.content_length = int(self.headers.get('content-length', '0'))
        self.headers_complete = True
        self.state = 'waiting'
        return
    
    def state_waiting(self):
        pass
    
    def read_body(self, cb, args = []):      
        self.body_cb = cb, args
        self.state = 'body'
        self.remaining_content = self.content_length
        if self.remaining_content == 0:
            self.state = "completed"
            return self.state_completed()
        self.conn.read_body()
   
    def read_body_stream(self, stream_cb, args=[]):
        self.remaining_content = self.content_length
        self.body_stream_cb = stream_cb, args
        self.state = 'body'
        self.conn.read_body()
        return self.state_body()
        
    def state_body(self):
        if self.body_stream_cb:
            bytes_available = min(len(self.conn.buffer), self.remaining_content)
            self.remaining_content -= bytes_available
            cb, args = self.body_stream_cb
            cb(self.conn.buffer.part(0,bytes_available), *args)
            self.conn.buffer.move(bytes_available)
        # Quick hack to fix body bug. TODO: clean up this whole function.
        elif len(self.conn.buffer) >= self.content_length:
            self.remaining_content = 0
        if self.remaining_content == 0:            
            self.state = 'completed'
            return self.state_completed()
        
    def state_completed(self):
        if self.body_stream_cb:
            cb, args = self.body_stream_cb
            cb("", *args)
        elif self.body_cb:
            cb, args = self.body_cb
            if cb:
                cb(self.conn.buffer.part(0, self.content_length), *args)
            self.conn.buffer.move(self.content_length)
        if len(self.conn.buffer) > 0:
            print "buffer: '%s'" % (self.conn.buffer.get_value(),)
            raise HTTPProtocolError, "Unexpected Data: %s" % (self.conn.buffer.get_value())
        
        self.state = 'write'
        for (mode, data, cb, args, eb, ebargs) in self.pending_actions:
            if mode == "write":
                self.write(data, cb, args, eb, ebargs, override=True)
            elif mode == "end":
                self.end(cb)
            elif mode == "close":
                self.close(cb)
                
    def write(self, data, cb=None, args=[],eb=None, ebargs=[],override=False):
        if self.write_ended and not override:
            raise Exception, "end already called"
        if self.state != 'write':
            self.pending_actions.append(("write", data, cb, args, eb, ebargs))
            if self.state == 'waiting':
                self.state = 'body'
                return self.state_body()
            return
        if len(data) == 0:
            return cb()
        self.write_queue_size += 1
        self.conn.write(data , self.write_cb, (cb, args), eb, ebargs)
    
    def write_cb(self, *args):
        self.write_queue_size -= 1
        if self.write_ended and self.write_queue_size == 0:
            if self.send_close:
                self.conn.close()
            else:
                self.conn.start_request()
        if len(args) > 0 and args[0] is not None:
            cb = args[0]
            cbargs = None
            if len(args) > 1:
                cbargs = args[1]
            if cbargs is None:
                cbargs = []
            cb(*cbargs)
            
    def end(self, cb=None, args=[]):
        if self.write_ended:
            raise Exception, "end already called"
        if self.state != "write":
            self.pending_actions.append(("end", None, cb, None, None, None))
            return
        self.write_ended = True
        self.write_queue_size +=1
        self.conn.write("", self.write_cb, (cb,))

    def close(self, cb=None, args=[]):
        if self.write_ended:
            raise Exception, "end already called"
        if self.state != "write":
            self.pending_actions.append(("close", None, cb, None, None, None))
            return
        if not self.write_ended:
            self.end(cb)
        self.send_close = True
    
    def close_now(self, reason="hard close"):
        self.conn.close(reason)



from client import SocketClient, SimpleClient
from server import SocketDaemon
from controller import SocketController
from connection import Connection
from websocket import WebSocketDaemon, WebSocketProxy, startwebsocketproxy



import event
from dez.http.server import HTTPDaemon, HTTPResponse, RawHTTPResponse

def test_dispatch(request):
    # Very raw way
#    request.write("HTTP/1.0 200 OK\r\nContent-type: text/html\r\nContent-length: 10\r\n\r\nAloha des!", None)
#    request.close()
    
    # HTTPReponse method
    r = HTTPResponse(request)
    r.write('Aloha Des!!')
    r.dispatch()

    # RawHTTPResponse method

#    r = RawHTTPResponse(request)
#    r.write_status(200, "OK")
#    r.write_header("good", "day")
#    r.write_header("Content-length", 10)
#    r.write_headers_end()
#    r.write('Aloha Des!')
#    r.close()


#
    # WSGIHTTPResponse method
    
#def simple_app(environ, start_response):
#    """Simplest possible application object"""
#    status = '200 OK'
#    response_headers = [('Content-type','text/plain')]
#    start_response(status, response_headers)
#    return ['Hello world!\n']
    
#   r = WSGIHTTPResponse(req, simple_app)
#   r.dispatch()

def noop():
    return True

def main(**kwargs):
    httpd = HTTPDaemon(kwargs['domain'], kwargs['port'])
    httpd.register_prefix("/index", test_dispatch)
    event.timeout(1, noop)
    event.dispatch()

def profile(**kwargs):
    import hotshot
    prof = hotshot.Profile("orbited_http_test.profile")
    prof.runcall(main, **kwargs)
    prof.close()



import event
from dez import io
from dez.network.connection import Connection

class SocketDaemon(object):
    def __init__(self, hostname, port, cb=None, b64=False, cbargs=[]):
        self.hostname = hostname
        self.port = port
        self.sock = io.server_socket(self.port)
        self.cb = cb
        self.cbargs = cbargs
        self.b64 = b64
        self.listen = event.read(self.sock, self.accept_connection)

    def accept_connection(self):
        sock, addr = self.sock.accept()
        conn = Connection(addr, sock, b64=self.b64)
        if self.cb:
            self.cb(conn, *self.cbargs)
        return True

    def start(self):
        event.signal(2, event.abort)
        event.dispatch()



class OPValidator(object):
    def __init__(self):
        self.actions = {
            "CONNECT": ['id','connection_id'],
            "SEND": ['id','recipients'],
            "WELCOME": ['id','user_key'],
            "UNWELCOME": ['id','user_key'],
            "CALLBACK": ['id','function'],
        }

    def __call__(self, req):
        if req.problem:
            req.error(req.problem)
            return False
        if req.action not in self.actions:
            req.error("invalid action:%s"%req.action)
            return False
        for header in self.actions[req.action]:
            if header not in req.headers:
                req.error("'%s' header missing from '%s' action"%(header,req.action))
                return False
        req.request_id = req.headers["id"]
        return True


class STOMPFrame(object):
    def __init__(self, conn, buffer):
        self.conn = conn
        self.request = buffer.data
        self.action = None
        self.headers = {}
        self.body = None
        try:
            self.problem = "error parsing action"
            tbreak = buffer.find('\n')
            self.action = buffer.part(0,tbreak)
            buffer.move(tbreak+1)
            self.problem = "error parsing headers"
            hbreak = buffer.find('\n\n')
            if hbreak != -1:
                self.headers = {}
                for header in buffer.part(0,hbreak).split('\n'):
                    key, val = header.split(': ')
                    self.headers[key] = val
                buffer.move(hbreak+2)
            self.problem = "error parsing body"
            self.body = buffer.data
            self.problem = None
        except:
            pass

    def connected(self, s):
        self.respond("CONNECTED",{"session":str(s)})

    def error(self, problem, body=""):
        self.respond("ERROR",{"message":problem},body,close=True)

    def received(self, rid, cb=None, cbarg=[]):
        if "receipt" in self.headers:
            self.respond("RECEIPT",{"receipt-id":rid},"",cb,cbarg)

    def respond(self, action, headers={}, body="", cb=None, args=[], close=False):
        self.conn.write(action,headers,body,cb,args,close)

    def close(self):
        self.conn.close()

class STOMPSendFrame(STOMPFrame):
    def __parse_headers(self, recips):
        s = ""
        for recip in recips:
            s += "recipient:"+str(recip)+"\r\n"
        return s

    def success(self, succ):
        self.conn.message("success",self.__parse_headers(succ))

    def failure(self, fail):
        self.conn.message("failure",self.__parse_headers(fail))



from dez.network import SocketClient, SocketDaemon, Connection
import sys

def main2(*args,**kwargs):
    client = SocketClient(cb=LocalEchoHTTPClient)
    client.connect(kwargs['domain'],kwargs['port'])

def delim_period(**kwargs):
    def new_conn(conn):
        EchoConnection(conn, '.')
    server = SocketDaemon(kwargs['domain'], kwargs['port'], new_conn)
    server.start()

def main(**kwargs):
    server = SocketDaemon(kwargs['domain'], kwargs['port'], cb=EchoChunked)
    server.start()

def delim_linebreak(**kwargs):
    server = SocketDaemon(kwargs['domain'], kwargs['port'], cb=EchoConnection)
    server.start()

def http(**kwargs):
    server = SocketDaemon(kwargs['domain'], kwargs['port'], cb=EchoHTTP)
    server.start()

class EchoChunked(object):
    def __init__(self, conn):
        self.conn = conn
        self.conn.set_rmode_close_chunked(self._recv)

    def _recv(self, data):
        print 'received:',data
        self.conn.write(data)

class EchoConnection(object):
    def __init__(self, conn, delim='\r\n'):
        self.delim = delim
        self.conn = conn
        self.conn.set_rmode_delimiter(self.delim, self.line_received)

    def line_received(self, data):
        self.conn.write("S: " + data + self.delim)
        print data
        #self.conn.set_rmode_close(self.line_received)

class LocalEchoHTTPClient(object):
    def __init__(self, conn):
        self.conn = conn
        self.conn.write("GET / HTTP/1.0\r\nHost: www.google.com\r\n\r\n")
        self.conn.set_rmode_delimiter('\r\n', self.line_received)
        
    def line_received(self, data):
        if data == "": # end of headers
            print "[end of headers]"
            self.conn.set_rmode_close(self.body)
        else:
            print data
            
    def body(self, data):
        print data
        sys.exit(0)

class EchoHTTP(object):
    def __init__(self, conn):
        self.conn = conn
        self.conn.set_rmode_delimiter('\r\n\r\n', self.line_received)

    def line_received(self, data):
        self.conn.write("HTTP/1.0 200 OK\r\nContent-Length: 0\r\n\r\n", self.conn.close)

class LengthConnection(object):
    def __init__(self, conn):
        self.conn = conn
        self.conn.set_rmode_delimiter('\r\n', self.length_received)

    def length_received(self, data):
        try:
            length = int(data)
        except:
            self.conn.write('Invalid integer: %s'%data, self.close)
        else:
            self.conn.set_rmode_size(length,self.body_received)

    def body_received(self, body):
        self.conn.write('s:%s'%body,self.close)

    def close(self):
        self.conn.close()


from setuptools import setup
setup(
    name='dez',
    version='0.5.0',
    author='Mario Balibrera',
    author_email='mario.balibrera@gmail.com',
    download_url='http://code.google.com/p/dez/downloads/list',
    license='MIT License',
    description='A set of pyevent-based network services',
    long_description='The dez library includes an asynchronous server benchmarking toolset; advanced, inotify-based static caching; XML and JSON stream parsing; Stomp, OP, HTTP, and WebSocket servers; and WebSocketProxy, which enables web application deployment on existing, unmodified TCP servers without straying from the HTML5 spec. In addition, dez offers a highly sophisticated API for implementing custom protocols, as well as a controller class that simplifies the creation of applications that require multiple servers to run in the same process.',
    packages=[
        'dez',
        'dez.http',
        'dez.http.server',
        'dez.http.client',
        'dez.http.proxy',
        'dez.stomp',
        'dez.stomp.server',
        'dez.op',
        'dez.op.server',
        'dez.network',
        'dez.samples'
    ],
    zip_safe = False,
    install_requires = [
        "rel >= 0.3.1",
        "demjson"
    ],
    entry_points = '''
        [console_scripts]
        dez_test = dez.samples.test:main
        dbench = dez.samples.http_load_test:main
        dez_websocket_proxy = dez.network:startwebsocketproxy
        dez_reverse_proxy = dez.http.reverseproxy:startreverseproxy
        [paste.server_runner]
        wsgi_server = dez.http.application:serve_wsgi_application
    ''',
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)



try:
    import cStringIO as StringIO
except ImportError:
    import StringIO

class HTTPResponse(object):
    id = 0
    def __init__(self, request):
        HTTPResponse.id += 1

        self.id = HTTPResponse.id
        self.request = request
        self.headers = {
            'Content-type': 'text/html'
        }
        self.buffer = []
        self.status = "200 OK"
        self.version_major = 1
        self.version_minor = min(1, request.version_minor)
        if self.version_minor == 1:
            self.headers['Connection'] = 'keep-alive'
            self.headers['Keep-alive'] = '300'

    def __setitem__(self, key, val):
        self.headers[key] = val

    def __getitem__(self, key):
        return self.headers[key]

    def write(self, data):
        self.buffer.append(str(data))

    def dispatch(self, cb=None):
#        print "DISPATCHING:", self.id
        status_line = "HTTP/%s.%s %s\r\n" % (
            self.version_major, self.version_minor, self.status)
        self.headers['Content-length'] = str(sum([len(s) for s in self.buffer]))
        h = "\r\n".join(": ".join((k, v)) for (k, v) in self.headers.items())
        h += "\r\n\r\n"
        response = status_line + h + "".join(self.buffer)
        self.request.write(response, None)
        if self.version_minor == 1 and int(self.headers.get('Keep-alive', '0')) > 0:
#            def cb(*args):
#                print 'ended!'
            self.request.end(cb)#cb)
        else:
#            print 'closing...'
            self.request.close(cb)

class HTTPVariableResponse(object):
    id = 0
    def __init__(self, request):
        HTTPResponse.id += 1
        self.id = HTTPResponse.id
        self.request = request
        self.started = False
        self.headers = {
            'Content-type': 'text/html'
        }
        self.status = "200 OK"
        self.version_major = 1
        self.version_minor = min(1, request.version_minor)
        if self.version_minor == 1:
            self.headers['Connection'] = 'keep-alive'
            self.headers['Keep-alive'] = '300'
            self.headers['Transfer-encoding'] = 'chunked'

    def __setitem__(self, key, val):
        self.headers[key] = val

    def __getitem__(self, key):
        return self.headers[key]

    def write(self, data, cb=None, args=[]):
        if not self.started:
            self.__start_response()
        if not data:
            return
        if self.version_minor == 1:
            self.__write_chunk(data, cb, args)
        else:
            self.request.write(data, cb, args)

    def __write_chunk(self, data, cb=None, args=[]):
        self.request.write("%X\r\n%s\r\n"%(len(data),data))
        if cb:
            cb(*args)

    def __start_response(self, cb=None):
        self.started = True
        status_line = "HTTP/%s.%s %s\r\n" % (
            self.version_major, self.version_minor, self.status)
        h = "\r\n".join(": ".join((k, v)) for (k, v) in self.headers.items())
        h += "\r\n\r\n"
        response = status_line + h
        self.request.write(status_line + h, None)

    def end(self, cb=None):
        if self.version_minor == 1:
            self.__write_chunk("")
            if int(self.headers.get('Keep-alive', '0')) > 0:
                return self.request.end(cb)
        self.request.close(cb)

class RawHTTPResponse(object):
    def __init__(self, request):
        self.request = request
        self.version_major = 1
        self.version_minor = min(1, request.version_minor)

    def write_status(self, code, reason, cb=None, args=[]):
        self.request.write("HTTP/%s.%s %s %s\r\n" % (
            self.version_major, self.version_minor, code, reason), cb, args)

    def write_header(self, key, value, cb=None, args=[]):
        self.request.write('%s: %s\r\n' % (key, value), cb, args)

    def write_headers_end(self, cb=None, args=[]):
        self.request.write('\r\n', cb, args)

    def write(self, data, cb=None, args=[],eb=None, ebargs=[]):
        self.request.write(data, cb, args, eb, ebargs)

    def write_chunk(self, data, cb=None, args=[]):
        self.request.write("%X\r\n%s\r\n"%(len(data),data))
        if len(data) == 0:
            return self.end()
        if cb:
            cb(*args)
    
    def close(self, cb=None):
        self.request.close(cb)

    def end(self, cb=None):
        self.request.end(cb)

class WSGIResponse(object):
    
    def __init__(self, request, app, host=None, port=None):
        self.request = request
        self.app = app
        self.host = host
        self.port = port
        self.response = HTTPResponse(self.request)
        self.environ = {}
        self.stderror = StringIO.StringIO()
        
        self.start_response_called = False
        self.headers_sent = False
        
    def dispatch(self):
#        print 'DEZ: reading body...'
        self.request.read_body(self.body_cb)

    def body_cb(self, body):
        """ response body transmission happens here
            
            Per PEP 333, we ensure that the headers have not already
            been sent. (which is a little silly since we are sending
            the entire response at once)
        """
#        print 'DEZ: Read post body:', body
#        print 'body_cb'
        self.setup_environ(body)
        output = self.app(self.environ, self.start_response)
        output = iter(output)
        
        try:
            first_iteration = output.next()
        except StopIteration:
            first_iteration = ""
        if self.headers_sent:
            raise AssertionError("start_response was not called")
        
        self.response.write(first_iteration)
        for data in output:    
            self.response.write(data)
        self.response.dispatch()

    def start_response(self, status, headers, exc_info=None):
        """ implements start_response per PEP 333
            
            We enforce:
                - if exception info has been sent, then ensure that
                  the headers have not already been sent.
                - start_response has not already been called.
            
        """
        
        if exc_info:
            try:
                if self.headers_sent:
                    raise exc_info[0],exc_info[1],exc_info[2]
            finally:
                exc_info = None
        elif self.start_response_called:
            raise AssertionError("start_response was called twice")
        
#        print 'start_response'
#        print 'status',status
#        print 'headers',headers
        self.response.status = status
        self.response.headers.update(headers)

    def setup_environ(self, body):
#        print 'setup_environ'
        environ = self.environ
        request = self.request
        environ['REMOTE_ADDR'] = self.request.conn.addr # Django fix
        environ['SERVER_NAME'] = self.host
        environ['SERVER_PORT'] = self.port
        environ['REQUEST_METHOD'] = request.method
        path, qs = request.url, ''
        if '?' in request.url:
            path, qs = path.split('?', 1)
#        path_info, script_name = path.rsplit('/', 1)
#        path_info += '/'
#        if request.url != '/favicon.ico':
#            print '==', request.url, path, qs
        environ['SCRIPT_NAME'] = ""#'%s:%s'%(self.host,self.port)
        environ['PATH_INFO'] = path
        environ['QUERY_STRING'] = qs
        content_type = request.headers.get('content-type', None)
        if content_type:
            environ['CONTENT_TYPE'] = content_type
#        content_length = request.headers.get('content-length', None)
#        content_length = len(body)
        environ['CONTENT_LENGTH'] = len(body)
        environ['wsgi.url_scheme']    = 'http'
        environ['wsgi.input']        = StringIO.StringIO(body)
        environ['wsgi.errors']       = self.stderror
        environ['wsgi.version']      = (1,1) # TODO: use version_minor and version_major
        environ['wsgi.multithread']  = False
        environ['wsgi.multiprocess'] = False
        environ['wsgi.run_once']     = False
        for key, val in self.request.headers.items():
            environ['HTTP_%s' % (key.upper().replace('-', '_'),)] = val
            



import urllib
from dez.buffer import Buffer
from dez.op.server.frame import Frame, SendFrame
from dez.http.client import HTTPClient

#DELIM = "^@\r\n"
DELIM = "\x00"

class Callback_Handler(object):
    def __init__(self, writer):
        self.writer = writer
        self.http = HTTPClient()
        self.cbs = {
            "signon":(False, None),
            "signoff":(False, None),
            "failure":(False, None),
            "success":(False, None),
            "message":(False, None)
        }

    def set_url(self, func, url):
        if func in self.cbs:
            self.cbs[func] = (True, url)
            return True
        return False

    def dispatch(self, func, headers, cb=None, cbarg=()):
        active, url = self.cbs[func]
        if active:
            headers["function"] = func
            if not url:
                return self.writer("CALLBACK", headers, "", cb, cbarg)
            final_headers = []
            for key in headers:
                if key == 'recipients':
                    for recipient in headers['recipients']:
                        final_headers.append(('recipient', recipient))
                else:
                    final_headers.append((key, headers[key]))
            self.http.get_url(url, "POST", {}, cb, cbarg, body=urllib.urlencode(final_headers))

class OPConnection(object):
    id = 0
    def __init__(self, conn, validator):
        OPConnection.id += 1
        self.id = OPConnection.id
        self.conn = conn
        self.validator = validator
        self.ids = set()
        self.app_cb = self.__default_app_cb
        self.app_args = []
        self.callbacks = Callback_Handler(self.write)
        self.conn.set_rmode_delimiter(DELIM, self.__connect)

    def __default_app_cb(self, req):
        print "no application callback specified"

    def set_request_cb(self, cb, args=()):
        self.app_cb = cb
        self.app_args = args

    def signon_cb(self, key, cb=None, cbarg=()):
        self.callback("signon",{"key":key},cb,cbarg)

    def signoff_cb(self, key, reason="", failed_msgs={}, cb=None, cbarg=()):
        headers = {"key":key}
        if reason:
            headers['reason'] = reason
        failed = ""
        for item in failed_msgs:
            failed += key + ":" + failed_msgs[item] + "|"
        if failed:
            headers["failed_msgs"] = failed[:-1]
        self.callback("signoff",headers,cb,cbarg)

    def message_cb(self, key, msg, cb=None, cbarg=()):
        self.callback("message",{"key":key,"msg":msg},cb,cbarg)

    def callback(self, func, headers, cb=None, cbarg=()):
        self.callbacks.dispatch(func, headers, cb, cbarg)

    def set_close_cb(self, cb, args=()):
        self.conn.set_close_cb(cb, args)

    def __parse_headers(self, headers):
        s = ""
        for key, val in headers.items():
            if key == "recipients":
                for r in val:
                    s += "recipient:"+str(r)+"\r\n"
            else:
                s += key+":"+val+"\r\n"
        return s

    def __connect(self, data):
        req = Frame(self, Buffer(data, mode="consume"))
        if self.validator(req):
            if req.action != "CONNECT":
                return req.error("'CONNECT' not sent")
            self.response = req.headers.get("response","none")
            self.__forward(req)
            self.conn.set_rmode_delimiter(DELIM, self.__process)

    def __process(self, data):
        req = Frame(self, Buffer(data, mode="consume"))
        if req.action == "CONNECT":
            return req.error("'CONNECT' already sent")
        if self.validator(req):
            self.__forward(req)

    def __forward(self, req):
        if req.request_id in self.ids:
            return req.error("non-unique 'id':%s"%req.request_id)
        self.ids.add(req.request_id)
        if "response" not in req.headers:
            req.headers["response"] = self.response
        if self.__final_check(req):
            self.app_cb(req)

    def __final_check(self, req):
        if req.action == "SEND":
            req.__class__ = SendFrame
        elif req.action == "CALLBACK":
            if not self.callbacks.set_url(req.headers['function'],req.headers.get('url',None)):
                req.error("Invalid function specified. Legitimate functions: signon, signoff, failure, success, message.")
                return False
        return True

    def write(self, action, headers, body, cb, cbarg):
        s = action+"\r\n"
        s += self.__parse_headers(headers)
        s += "\r\n"+body+DELIM
        self.conn.write(s, cb, cbarg)

    def close(self):
        self.conn.close()



from xml.dom.minidom import parseString

AMP_ESCAPE = "__AMP_ESCAPE__"

class XMLNode(object):
    def __init__(self, name):
        self.name = name
        self.attributes = {}
        self.children = []

    def __repr__(self):
        return "<XMLNode %s>"%self.name

    def __str__(self):
        if self.children:
            return "<%s>%s</%s>"%(self._full_name(), ''.join([str(c) for c in self.children]), self.name)
        return "<%s/>"%(self._full_name())

    def __len__(self):
        return len(self.__str__())

    def _full_name(self):
        if self.attributes:
            return self.name + ' ' + ' '.join(["%s='%s'"%(key, val) for key, val in self.attributes.items()])
        return self.name

    def has_children(self):
        return bool(self.children)

    def has_attribute(self, attr):
        return attr in self.attributes

    def attr(self, attr):
        return self.attributes.get(attr, None)

    def add_child(self, child):
        self.children.append(child)

    def add_attribute(self, key, val):
        self.attributes[key] = val

def new_node(dnode):
    if dnode.nodeType == dnode.TEXT_NODE:
        return dnode.data.replace(AMP_ESCAPE, "&")
    node = XMLNode(dnode.nodeName)
    for key, val in dnode.attributes.items():
        node.add_attribute(key, val)
    for child in dnode.childNodes:
        node.add_child(new_node(child))
    return node

def extract_xml(string):
    try:
        return new_node(parseString(string.replace("&",AMP_ESCAPE)).firstChild)
    except:
        return None


import os
import sys

from dez.http.application import HTTPApplication
import django.core.handlers.wsgi

# Set the path where the django wsgi handler can find the "djangochat" module
# alternatively, comment out below line, and just run this script from the
# same path as the django module.
sys.path.insert(0, '/home/mario/stuff/__orbited/demos/python')


def main(**kwargs):
    # the name of the django app is "djangochat"
    os.environ["DJANGO_SETTINGS_MODULE"] = "djangochat.settings"
    application = django.core.handlers.wsgi.WSGIHandler()    
    server = HTTPApplication("", kwargs['port'])
    
    # forward any traffic starting with '/' to the application
    # Note: in this case, that means all urls are sent to django
    server.add_wsgi_rule('/', application)
    # go
    server.start()



from dez.http.client import HTTPClientRequest, HTTPClient
import event

def main(**kwargs):
    client = HTTPClient()
    client.get_connection(kwargs['domain'], kwargs['port'], get_conn_cb, [])
    event.dispatch()
    
def get_conn_cb(conn):
    print 'GOT conn', conn
    req = HTTPClientRequest(conn)
#    req.headers['foo'] = 'bar'
#    req.method = 'get'
#    req.write('?baz=foobaz&bar=foobar')
    req.dispatch(response_headers_end_cb, [])


def response_headers_end_cb(response):
    response.read_body(response_completed_cb, [response])

def alternate_response_headers_end_cb(response):
    print 'woot4'
    response.read_body_stream(response_body_stream_cb, [response])

def response_body_stream_cb(chunk, response):
    print chunk

def response_completed_cb(body, response):
    print 'woot5'
    print response.status_line
    print response.headers
    print "#########"
    print "body len:", len(body)
    event.abort()    



import event
from dez.network.server import SocketDaemon

class SocketController(object):
    def __init__(self):
        self.daemons = {}

    def register_address(self, hostname, port, callback, cbargs=[], b64=False):
        if (hostname, port) in self.daemons:
            self.daemons[(hostname, port)].cb = callback
            self.daemons[(hostname, port)].cbargs = cbargs
        else:
            self.daemons[(hostname, port)] = SocketDaemon(hostname, port, callback, b64, cbargs)

    def start(self):
        if not self.daemons:
            print "SocketController doesn't know where to listen. Use register_address(hostname, port, callback) to register server addresses."
            return
        event.signal(2, event.abort)
        event.dispatch()



from dez.http.client import HTTPClient
import event
    
def main(**kwargs):
    #url = "http://www.google.com:80/search?q=orbited"
    url = "http://%s:%s/"%(kwargs['domain'],kwargs['port'])
    c = HTTPClient()
    c.get_url(url, cb=req_cb, timeout=1)
    event.signal(2, event.abort)
    event.dispatch()

def req_cb(response):
    print response.status_line



import event
from dez import io
from dez.network.connection import Connection

class SimpleClient(object):
    def __init__(self, b64=False):
        '''If b64=True, the client reads and writes base64-encoded strings'''
        self.b64 = b64

    def connect(self, host, port, cb, args=[]):
        sock = io.client_socket(host, port)
        self.conn = Connection((host, port), sock, b64=self.b64)
        cb(self.conn, *args)
        event.signal(2, event.abort)
        event.dispatch()

class SocketClient(object):
    def __init__(self):
        self.pools = {}

    def get_connection(self, host, port, cb, args=[], eb=None, ebargs=None, timeout=60, max_conn=5, b64=False):
        addr = host, port
        if addr not in self.pools:
            self.pools[addr] = ConnectionPool(host, port, max_conn, b64)
        self.pools[addr].get_connection(cb, args, timeout)

    def start_connections(self, host, port, num, cb, args=[], timeout=None, max_conn=5):
        addr = host, port
        if addr not in self.pools:
            self.pools[addr] = ConnectionPool(host, port, max_conn)
        self.pools[addr].start_connections(num, cb, args, timeout)

#    def free_connection(self, conn):
#        conn.__start()
#        self.pools[conn.addr].connection_available(conn)

#    def connect(self,hostname,port,cb=None):
#        s = io.client_socket(hostname,port)
#        conn = Connection((hostname,port), s)
#        #TODO pass the callback on to Connection, don't cal it here
#        cb(conn)


class ConnectionPool(object):
    def __init__(self, hostname, port, max_connections=5, b64=False):
        self.addr = hostname, port
        self.connection_count = 0
        self.max_connections = max_connections
        self.b64 = b64

        # real connections
        self.pool = []
#        self.in_use = []

        # requests for connections
        self.wait_index = 0
        self.wait_queue = []
        self.wait_timers = {}
        self.__start_cb_info = None
        self.__start_timer = None
        self.__start_count = None
        
    def start_connections(self, num, cb, args, timeout=None):
        if self.__start_cb_info:
            raise Exception("StartInProgress"), "Only issue one start_connections call in parellel"
        if timeout:
            self.__start_timer = event.timeout(timeout, __start_timeout_cb)
        self.__start_cb_info = (cb, args)
        self.__start_count = num
        for i in range(num):
            self.__start_connection()
        
    def get_connection(self, cb, args, timeout):
        if self.pool:
            c = self.pool.pop()
#            self.in_use.append(c)
            return cb(c,*args)

        if self.connection_count < self.max_connections:
            # make a new connection
#            print 'open a new connection'
            self.__start_connection()

        i = self.wait_index
        timer = event.timeout(timeout, self.__timed_out, i)
        self.wait_timers[i] = cb, args, timer
        self.wait_queue.append(i)
        self.wait_index += 1

        self.__service_queue()

    def __start_connection(self):
            sock = io.client_socket(*self.addr)
            conn = Connection(self.addr, sock, self, self.b64)
            conn.connect()
            self.connection_count += 1

    def connection_available(self, conn):
        self.pool.append(conn)
#        print 'conn available', len(self.pool)
        if self.__start_count and len(self.pool) == self.__start_count:          
            cb, args = self.__start_cb_info
            self.__start_cb_info = None
            if self.__start_timer:
                self.__start_timer.delete()
                self.__start_timer = None
            self.__start_count = None
            cb(*args)
        self.__service_queue()        

    def connection_closed(self, conn):
        if conn in self.pool:
            self.pool.remove(conn)
        self.connection_count -= 1
        if self.wait_queue:
            self.__start_connection()
        self.__service_queue()

    def __timed_out(self, i):
        cb, args, timer = self.wait_timers[i]
        timer.delete()
        del self.wait_timers[i]
        self.wait_queue.remove(i)
        self.connection_count -= 1

    def __service_queue(self):
        if self.pool and self.wait_queue:
            i = self.wait_queue.pop()
            cb, args, timer = self.wait_timers.pop(i)
            timer.delete()
            
            c = self.pool.pop()
#            self.in_use.append(c)
            cb(c, *args)
            



from dez.network.server import SocketDaemon
from dez.op.server.validator import OPValidator
from dez.op.server.connection import OPConnection

class OPServer(object):
    def __init__(self, addr, port):
        self.__val = OPValidator()
        self.__server = SocketDaemon(addr, port, self.__connect_cb)
        self.__app_connect_cb = None

    def start(self):
        self.__server.start()

    def set_connect_cb(self, cb):
        self.__app_connect_cb = cb

    def __connect_cb(self, c):
        if self.__app_connect_cb:
            return self.__app_connect_cb(OPConnection(c, self.__val))
        print "Application server not available"



"""
Select a JSON library from any of several known libraries.
"""

try:
    import cjson
    encode = cjson.encode
    decode = cjson.decode
except ImportError:
    try:
        import simplejson
        encode = simplejson.dumps
        decode = simplejson.loads
    except ImportError:
        try:
            import demjson
            encode = demjson.encode
            decode = demjson.decode
        except ImportError:
            raise ImportError, "could not load one of: cjson, simplejson, demjson"



import time, event
from dez.network.client import SocketClient
from dez.http.client import HTTPClientRequest, HTTPClient
from dez.http.server import HTTPDaemon, HTTPResponse, RawHTTPResponse
from dez.http.client.response import HTTPClientReader

client = SocketClient()

def proxy(request):
    client.get_connection("www.orbited.org", 80, connection_cb, [request])

def connection_cb(conn, request):
    conn.write(request.action + '\r\n')
    for (key, val) in request.headers.items():
        conn.write('%s: %s\r\n' % (key, val))
    conn.write('\r\n')
    request.read_body_stream(body_cb, [conn, request])

def body_cb(data, conn, request):
    if not data:
        reader = HTTPClientReader(conn)
        reader.get_headers_only(response_headers_cb, [reader, request])
    else:    
        conn.write(data)

def response_headers_cb(response, reader, request):
    request.write(response.status_line + '\r\n')
    for (key, val) in response.case_match_headers.items():
        request.write('%s: %s\r\n' % (val, response.headers[key]))
    request.write('\r\n')
    reader.get_body_stream(response_body_cb, [ request])
    
def response_body_cb(response, size, request):
#    print 'body', size
    if size == 0:
        print 'request.end'
        return request.end()
#    print 'write:', len(response.body.get_value())
#    def cb(*args):
#        print 'wrote!', args
    request.write(response.body.get_value())
    response.body.reset()
        
def main(**kwargs):
    event.signal(2, event.abort)
    server = HTTPDaemon("", kwargs['port'])
    server.register_prefix("/", proxy)
    event.dispatch()
